# vim:sw=4:et

import time
from gaphor.misc.xmlwriter import XMLWriter

class XMLAttributes(dict):
    def getLength(self):
        """ Return the number of attributes. """
        return len(self.keys())

    def getNames(self):
        """ Return the names of the attributes. """
        return self.keys()
        
    def getType(self, name):
        """ Returns the type of the attribute name, which is normally 'CDATA'. """
        return 'CDATA'

    def getValue(name):
        """ Return the value of attribute name. """ 
        return self[name]


class XMIExport(object):
    
    def __init__(self, element_factory):
        self.element_factory = element_factory

    # State diagram specific
    # ======================
    def handleInitialNode(self, xmi, node):
        attributes = XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['name']="start" # Gaphor doens't have name support 
                                   # for start actions.
        attributes['visibility'] = 'public'
        attributes['isSpecification'] = 'false'
        attributes['kind'] = 'initial'
        xmi.startElement('UML2:Pseudostate', attrs=attributes)
        xmi.startElement('UML2:Vertex.outgoing')
        
                         
        
    def handleAction(self, xmi, node):
        attributes = XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['name']=node.name
        attributes['visibility'] = 'public'
        attributes['isSpecification'] = 'false'
    
    def handleControlFlow(self, xmi, node):
        pass
    
    
    # Class diagram specific
    #=========================

    def handlePackage(self, xmi, node):
        if node.package != None: return
        self.writePackage(xmi, node)

    def writePackage(self, xmi, node):
        attributes=XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['name']=node.name
        attributes['visibility']='public' # TODO
        attributes['isSpecification']='false'
        attributes['isRoot']='false'
        attributes['isLeaf']='false'
        attributes['isAbstract']='false' #node.isAbstract and 'true' or 'false'
        attributes['isActive']='false'
        xmi.startElement('UML:Package', attrs=attributes)
        xmi.startElement('UML:Namespace.ownedElement', attrs=XMLAttributes())
        classes = [element for element in self.element_factory.select()]
        for item in classes:
            try:
                package = item.package==node
            except AttributeError:
                continue
            if package:
                try:
                    handler=getattr(self, 'write%s'%item.__class__.__name__)
                except AttributeError:
                    continue
                handler(xmi, item)
        xmi.endElement('UML:Namespace.ownedElement')
        xmi.endElement('UML:Package')


    def handleClass(self, xmi, node):
        """ """ 
        # Check for packages, classes will be serialized through the package
        if node.package != None: return
        
        self.writeClass(xmi, node)

    def writeClass(self, xmi, node):
        # Check for stereotype definition classes
        if node.name in ('Class', 'Interface', 'Package'): return
        
        attributes=XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['name']=node.name
        attributes['visibility']='public' # TODO
        attributes['isSpecification']='false'
        attributes['isRoot']='false'
        attributes['isLeaf']='false'
        attributes['isAbstract']=node.isAbstract and 'true' or 'false'
        attributes['isActive']='false'
        xmi.startElement('UML:Class', attrs=attributes)
        
        # Generalization stuff
        generalizations = node.generalization
        if generalizations:
            xmi.startElement('UML:GeneralizableElement.generalization', attrs=XMLAttributes())
            for item in generalizations:
                attributes=XMLAttributes()
                attributes['xmi.idref']=item.id
                xmi.startElement('UML:Generalization', attrs=attributes)
                xmi.endElement('UML:Generalization')
            xmi.endElement('UML:GeneralizableElement.generalization')
    
        # Stereotypes
        stereotypes = node.appliedStereotype
        if stereotypes:
            xmi.startElement('UML:ModelElement.stereotype', attrs=XMLAttributes())
            for stereotype in stereotypes:
                attributes = XMLAttributes({'xmi.idref': stereotype.id})
                xmi.startElement('UML:Stereotype', attrs=attributes)
                xmi.endElement('UML:Stereotype')
            xmi.endElement('UML:ModelElement.stereotype')
        
        # Generate the field type classes
        xmi.startElement('UML:Namespace.ownedElement', attrs=XMLAttributes())
        for attribute in node.ownedAttribute:
            try:
                attribute.name
                attribute.typeValue.value
            except AttributeError:
                continue
                
            attributes=XMLAttributes()
            attributes['xmi.id']=attribute.typeValue.id
            typeName = attribute.typeValue.value
            if not typeName:
                print "Warning, no type name given for attribute (so no export for this one)"
                continue
            attributes['name']=typeName
            attributes['visibility']='public'
            attributes['isSpecification']='false'
            attributes['isRoot']='false'
            attributes['isLeaf']='false'
            attributes['isAbstract']='false'
            attributes['isActive']='false'
            xmi.startElement('UML:Class', attrs=attributes)
            xmi.endElement('UML:Class')
        xmi.endElement('UML:Namespace.ownedElement')
 
        xmi.startElement('UML:Classifier.feature', attrs=XMLAttributes())
        
        # Now generate the XML for the attribute itself
        for attribute in node.ownedAttribute:
            try:
                attribute.name
                attribute.typeValue.value
            except AttributeError:
                continue
 
            attributes=XMLAttributes()
            attributes['xmi.id']=attribute.id
            attributes['name']=attribute.name or ''
            attributes['visibility']='public'
            attributes['isSpecification']='false'
            attributes['ownerScope']='instance'
            attributes['changeability']='changeable'
            xmi.startElement('UML:Attribute', attrs=attributes)
            
            xmi.startElement('UML:StructuralFeature.type', XMLAttributes())
            xmi.startElement('UML:Class', XMLAttributes({'xmi.idref': attribute.typeValue.id}))
            xmi.endElement('UML:Class')
            xmi.endElement('UML:StructuralFeature.type')
                           
            xmi.endElement('UML:Attribute')
        
        # Generate all methods
        for operation in node.ownedOperation:
            attributes=XMLAttributes()
            attributes['xmi.id']=operation.id
            attributes['name']=operation.name or ''
            attributes['visibility']='public'
            attributes['isSpecification']='false'
            attributes['ownerScope']='instance'
            attributes['isQuery']='false'
            attributes['concurrency']='sequential'
            attributes['isRoot']='false'
            attributes['isLeaf']='false'
            attributes['isAbstract']='false'
            xmi.startElement('UML:Operation', attrs=attributes)
            
            xmi.startElement('UML:BehavioralFeature.parameters', XMLAttributes())
            #self.writeParameters(operation.returnResult, operation, xmi, kind='return')
            self.writeParameters(operation.formalParameter, operation, xmi, kind='in')

            xmi.endElement('UML:BehavioralFeature.parameters')

            xmi.endElement('UML:Operation')

        
        xmi.endElement('UML:Classifier.feature')
        xmi.endElement('UML:Class')

    def writeParameters(self, parameters, operation, xmi, kind='in'):
        for parameter in parameters:
            attributes=XMLAttributes()
            attributes['xmi.id']=parameter.id
            attributes['name']=parameter.name or ''
            attributes['kind']=kind
            attributes['isSpecification']='false'
            xmi.startElement('UML:Parameter', attrs=attributes)
            xmi.startElement('UML:Parameter.type', attrs=XMLAttributes())
            xmi.startElement('UML:DataType', XMLAttributes({'xmi.idref': parameter.typeValue.id}))
            xmi.endElement('UML:DataType')
            xmi.endElement('UML:Parameter.type')
            xmi.endElement('UML:Parameter')


    def handleGeneralization(self, xmi, node):
        # Write out the generalization specifications
        attributes=XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['isSpecification']='false'
        xmi.startElement('UML:Generalization', attrs=attributes)
        xmi.startElement('UML:Generalization.child', attrs=XMLAttributes())
        attributes=XMLAttributes()
        attributes['xmi.idref']=node.specific.id
        xmi.startElement('UML:Class', attrs=attributes)
        xmi.endElement('UML:Class')
        xmi.endElement('UML:Generalization.child')
        xmi.startElement('UML:Generalization.parent', attrs=XMLAttributes())
        attributes=XMLAttributes()
        attributes['xmi.idref']=node.general.id
        xmi.startElement('UML:Class', attrs=attributes)
        xmi.endElement('UML:Class')
        xmi.endElement('UML:Generalization.parent')
        xmi.endElement('UML:Generalization')
        
    def handleStereotype(self, xmi, node):
        attributes=XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['name']=node.name
        attributes['visibility']='public'
        attributes['isSpecification']='false'
        attributes['isRoot']='false'
        attributes['isAbstract']='false'
        xmi.startElement('UML:Stereotype', attributes)
        xmi.startElement('UML:Stereotype.baseClass', attrs=XMLAttributes())
        xmi.characters('Classifier')
        xmi.endElement('UML:Stereotype.baseClass')
        xmi.endElement('UML:Stereotype')
        
    def handleAssociation(self, xmi, node):
        """
        """
        ends=node.memberEnd
        if len(ends)!=2:
            return
        
        """
        No longer in use as Gaphor is updated
        # Temp fix for line direction
        end_sequence = [0,1]
        if ends[1].aggregation=='composite':
            end_sequence = [0,1]
        else:
            end_sequence = [1,0]
        if ends[1].class_:
            end_sequence.reverse()
        """
        end_sequence = [0,1]

        attributes=XMLAttributes()
        attributes['xmi.id']=node.id
        attributes['isSpecification']='false'
        attributes['isRoot']='false'
        attributes['isLeaf']='false'
        attributes['isAbstract']='false'
        name = node.name
        if name:
            attributes['name']=name
        xmi.startElement('UML:Association', attrs=attributes)
        xmi.startElement('UML:Association.connection', attrs=XMLAttributes())

        for i in end_sequence: # Need an index for class lookup
            end=ends[i]
            attributes=XMLAttributes()
            attributes['xmi.id']=end.id
            attributes['visibility']='public'
            attributes['isSpecification']='false'
            attributes['isNavigable']=end.class_ and 'true' or 'false'
            attributes['ordering']='unordered'
            attributes['aggregation']=ends[1-i].aggregation # TODO: Handle None?
            attributes['targetScope']='instance' 
            attributes['changeability']='changeable'
            
            name = end.name
            if name:
                attributes['name']=name 

            xmi.startElement('UML:AssociationEnd', attrs=attributes)
            xmi.startElement('UML:AssociationEnd.multiplicity', attrs=XMLAttributes())
            attributes=XMLAttributes()
            attributes['xmi.id']=end.id+':%s'%i # Fabricate and id
            xmi.startElement('UML:Multiplicity', attrs=attributes)
            xmi.startElement('UML:Multiplicity.range', attrs=XMLAttributes())
            attributes=XMLAttributes()
            attributes['xmi.id']=str(id(attributes)) # No id in Gaphor for this
            values=('lower','upper')
            for value in values:
                try:
                    data=getattr(end, '%sValue'%value).value
                except AttributeError:
                    data='1' # FIXME!
                if str(data)=='*':
                    attributes['lower']='0'
                    attributes['upper']='-1'
                    break
                elif data is None:
                    data='1'
                attributes[value]=data
                
            xmi.startElement('UML:MultiplicityRange', attrs=attributes)
            xmi.endElement('UML:MultiplicityRange')
            xmi.endElement('UML:Multiplicity.range')
            xmi.endElement('UML:Multiplicity')
            xmi.endElement('UML:AssociationEnd.multiplicity')
            xmi.startElement('UML:AssociationEnd.participant', attrs=XMLAttributes())
            attributes=XMLAttributes()
            attributes['xmi.idref']=end.type.id 
            xmi.startElement('UML:Class', attrs=attributes)
            xmi.endElement('UML:Class')
            xmi.endElement('UML:AssociationEnd.participant')
            xmi.endElement('UML:AssociationEnd')
        xmi.endElement('UML:Association.connection')
        xmi.endElement('UML:Association')
            
            
    def export(self, filename):
        #out=open('/Users/vloothuis/test.xmi','w')
        out=open(filename,'w')

        xmi=XMLWriter(out)
        
        # Start XML generation
        attributes=XMLAttributes()
        attributes['xmi.version']='1.2'
        attributes['xmlns:UML']='org.omg.xmi.namespace.UML'
        attributes['timestamp'] = time.strftime('%a %b %d %H:%M:%S %Z %Y')
        xmi.startElement('XMI', attrs=attributes)
        
        self.writeHeader(xmi)
        
        # TODO: Add diagram stuff here
        
        self.writeModel(xmi, attributes)
        
        xmi.endElement('XMI')
        
        handlers={}

    def writeHeader(self, xmi):
        # Write out the XMI header
        xmi.startElement('XMI.header', attrs=XMLAttributes())
        xmi.startElement('XMI.documentation', attrs=XMLAttributes())
        xmi.startElement('XMI.exporter', attrs=XMLAttributes())
        xmi.characters('Gaphor XMI Export plugin')
        xmi.endElement('XMI.exporter')
        xmi.startElement('XMI.exporterVersion', attrs=XMLAttributes())
        xmi.characters('1.0') # TODO!
        xmi.endElement('XMI.exporterVersion')
        xmi.endElement('XMI.documentation')
        xmi.endElement('XMI.header')


    def writeModel(self, xmi, attributes):
        # Generator the model
        xmi.startElement('XMI.content', attrs=XMLAttributes())
        # Now write out the model
        attributes=XMLAttributes()
        attributes['xmi.id']='sm$f14318:ff442f5793:-7f6e'
        attributes['name'] = 'model 1'
        attributes['isSpecification'] = 'false'
        attributes['isRoot'] = 'false'
        attributes['isLeaf'] = 'false'
        attributes['isAbstract'] = 'false'
        xmi.startElement('UML:Model', attrs=attributes)
        xmi.startElement('UML:Namespace.ownedElement', attrs=XMLAttributes())
        for element in self.element_factory.select():
            #print element.__class__.__name__
            try:
                handler=getattr(self, 'handle%s'%element.__class__.__name__)
            except AttributeError:
                print element.__class__.__name__
                continue
            handler(xmi, element)
        xmi.endElement('UML:Namespace.ownedElement')    
        xmi.endElement('UML:Model')
        xmi.endElement('XMI.content')




#!/usr/bin/env python
# vim: sw=4:et

import sys
import gtk

from gaphor.application import Application
from gaphor.ui.abstractwindow import AbstractWindow

_WELCOME_TEXT = """Welcome in Gaphor's Editor!

Here you can create small scripts.
The following modules are already
imported:
gaphor - Gaphor package
UML - UML data model (gaphor.UML)
ui - User interface stuff (gaphor.ui)
diagram - Diagram items (gaphor.diagram)
misc - utility package (gaphor.misc)

For convenience, the ElementFactory
has been added as 'element_factory'.
---
"""

class EditorWindow(AbstractWindow):
    
    menu = ('_File', (
                'EditorRun',
                'separator',
                'FileClose'),
            '_Edit', (
                'EditorClear',),
            )
    toolbar = ('EditorRun', 'separator', 'EditorClear')

    def __init__(self):
        AbstractWindow.__init__(self)
        # Set up local variables for execution
        import gaphor
        import gaphor.UML
        import gaphor.ui
        import gaphor.misc
        import gaphor.diagram
        self.__locals = {
            'gaphor': gaphor,
            'UML': gaphor.UML,
            'ui': gaphor.ui,
            'misc': gaphor.misc,
            'diagram': gaphor.diagram,
            'element_factory': Application.get_service('element_factory'),
            'clear': self.clear_results
        }

    def get_source_text_view(self):
        self._check_state(AbstractWindow.STATE_ACTIVE)
        return self.source

    def get_result_text_view(self):
        self._check_state(AbstractWindow.STATE_ACTIVE)
        return self.result

    def construct(self):
        paned = gtk.VPaned()
        
        source = gtk.TextView()
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        scrolled_window.add(source)
        paned.pack1(scrolled_window, resize=True)

        result = gtk.TextView()
        result.set_editable(0)
        result.set_wrap_mode(gtk.WRAP_CHAR)
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        scrolled_window.add(result)
        paned.pack2(scrolled_window, resize=True)
        tb = result.get_buffer()
        tb.insert(tb.get_end_iter(), _WELCOME_TEXT)
        
        self.source = source
        self.result = result
        paned.show_all()

        self._construct_window(name='editor',
                               title='Gaphor Editor',
                               size=(400, 400),
                               contents=paned)

    def run(self):
        self._check_state(AbstractWindow.STATE_ACTIVE)
        orig_stdout = sys.stdout
        orig_stderr = sys.stderr
        result_buf = self.get_result_text_view().get_buffer()
        sys.stdout = PseudoFile(result_buf, 'stdout')
        sys.stderr = PseudoFile(result_buf, 'stderr')
        try:
            text_buf = self.get_source_text_view().get_buffer()
            text = text_buf.get_text (text_buf.get_start_iter(),
                                          text_buf.get_end_iter(), 0)

            code = None
            try:
                code = compile(text + '\n', '<input>', 'exec')
            except (SyntaxError, OverflowError, ValueError):
                print 'Error'
            if code:
                exec code in self.__locals
        except Exception, e:
            import traceback
            print 'ERROR: ',
            traceback.print_exc()
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__

    def clear_results(self):
        """Clear the results text buffer."""
        self._check_state(AbstractWindow.STATE_ACTIVE)
        tb = self.result.get_buffer()
        tb.delete(tb.get_start_iter(), tb.get_end_iter())

    def _on_window_destroy(self, window):
        AbstractWindow._on_window_destroy(self, window)
        del self.source
        del self.result


class PseudoFile:

    def __init__(self, text_buffer, tags):
        self.text_buffer = text_buffer
        self.tags = tags

    def write(self, s):
        #self.shell.write(s, self.tags)
        try:
            tb = self.text_buffer
            tb.insert(tb.get_end_iter(), s)
        except Exception, e:
            pass
        while gtk.events_pending():
            gtk.main_iteration(False)

    def writelines(self, l):
        map(self.write, l)

    def flush(self):
        pass

    def isatty(self):
        return 1

import editoractions




"""

This module is not yet ready to be used for adapters.

Here the logic should be placed that allows the application to draw already
existing relationships on the diagram when a model element is added (e.g.
by drag and drop).
"""

from zope import component


class AssociationRelationship(Relationship):
    """Relationship for associations.
    """
    def relationship(self, line, head_subject = None, tail_subject = None):
        # First check if we do not already contain the right subject:
        if line.subject:
            end1 = line.subject.memberEnd[0]
            end2 = line.subject.memberEnd[1]
            if (end1.type is head_type and end2.type is tail_type) \
               or (end2.type is head_type and end1.type is tail_type):
                return
                
        # Find all associations and determine if the properties on the
        # association ends have a type that points to the class.
        Association = UML.Association
        for assoc in resource(UML.ElementFactory).itervalues():
            if isinstance(assoc, Association):
                #print 'assoc.memberEnd', assoc.memberEnd
                end1 = assoc.memberEnd[0]
                end2 = assoc.memberEnd[1]
                if (end1.type is head_type and end2.type is tail_type) \
                   or (end2.type is head_type and end1.type is tail_type):
                    # check if this entry is not yet in the diagram
                    # Return if the association is not (yet) on the canvas
                    for item in assoc.presentation:
                        if item.canvas is line.canvas:
                            break
                    else:
                        return assoc
        return None


# vim:sw=4:et:ai




"""AssociationItem -- Graphical representation of an association.

Plan:
 - transform AssociationEnd in a (dumb) data class
 - for assocation name and direction tag, use the same trick as is used
   for line ends.
 - 

"""

# TODO: for Association.postload(): in some cases where the association ends
# are connected to the same Class, the head_end property is connected to the
# tail end and visa versa.

from math import atan2, pi

from gaphas.util import text_extents, text_align, text_multiline
from gaphas.state import reversible_property
from gaphas import Item
from gaphas.geometry import Rectangle, distance_point_point_fast
from gaphas.geometry import distance_rectangle_point, distance_line_point

from gaphor import UML
from gaphor.diagram.diagramitem import SubjectSupport
from gaphor.diagram.diagramline import NamedLine


class AssociationItem(NamedLine):
    """
    AssociationItem represents associations. 
    An AssociationItem has two AssociationEnd items. Each AssociationEnd item
    represents a Property (with Property.association == my association).
    """

    __uml__ = UML.Association

    def __init__(self, id=None):
        NamedLine.__init__(self, id)

        # AssociationEnds are really inseperable from the AssociationItem.
        # We give them the same id as the association item.
        self._head_end = AssociationEnd(owner=self, end="head")
        self._tail_end = AssociationEnd(owner=self, end="tail")

        # Direction depends on the ends that hold the ownedEnd attributes.
        self._show_direction = False
        self._dir_angle = 0
        self._dir_pos = 0, 0

    def set_show_direction(self, dir):
        self._show_direction = dir
        self.request_update()

    show_direction = reversible_property(lambda s: s._show_direction, set_show_direction)

    def setup_canvas(self):
        super(AssociationItem, self).setup_canvas()
        self._head_end._canvas = self.canvas
        self._tail_end._canvas = self.canvas

    def teardown_canvas(self):
        super(AssociationItem, self).teardown_canvas()
        del self._head_end._canvas
        del self._tail_end._canvas

    def save(self, save_func):
        NamedLine.save(self, save_func)
        save_func('show-direction', self._show_direction)
        if self._head_end.subject:
            save_func('head-subject', self._head_end.subject)
        if self._tail_end.subject:
            save_func('tail-subject', self._tail_end.subject)

    def load(self, name, value):
        # end_head and end_tail were used in an older Gaphor version
        if name in ( 'head_end', 'head_subject', 'head-subject' ):
            #type(self._head_end).subject.load(self._head_end, value)
            #self._head_end.load('subject', value)
            self._head_end.subject = value
        elif name in ( 'tail_end', 'tail_subject', 'tail-subject' ):
            #type(self._tail_end).subject.load(self._tail_end, value)
            #self._tail_end.load('subject', value)
            self._tail_end.subject = value
        else:
            NamedLine.load(self, name, value)

    def postload(self):
        NamedLine.postload(self)
        self._head_end.set_text()
        self._tail_end.set_text()

    head_end = property(lambda self: self._head_end)

    tail_end = property(lambda self: self._tail_end)

    def unlink(self):
        self._head_end.unlink()
        self._tail_end.unlink()
        NamedLine.unlink(self)

    def invert_direction(self):
        """Invert the direction of the association, this is done by
        swapping the head and tail-ends subjects.
        """
        if not self.subject:
            return

        self.subject.memberEnd.swap(self.subject.memberEnd[0], self.subject.memberEnd[1])
        self.request_update()

    def on_subject_notify(self, pspec, notifiers=()):
        NamedLine.on_subject_notify(self, pspec,
                notifiers + ('ownedEnd', 'memberEnd'))

    def on_subject_notify__ownedEnd(self, subject, pspec):
        self.request_update()

    def on_subject_notify__memberEnd(self, subject, pspec):
        self.request_update()

    def post_update(self, context):
        """
        Update the shapes and sub-items of the association.
        """

        handles = self.handles()

        # Update line endings:
        head_subject = self._head_end.subject
        tail_subject = self._tail_end.subject
        
        # Update line ends using the aggregation and isNavigable values:
        if head_subject and tail_subject:
            if tail_subject.aggregation == intern('composite'):
                self.draw_head = self.draw_head_composite
            elif tail_subject.aggregation == intern('shared'):
                self.draw_head = self.draw_head_shared
            elif self._head_end.get_navigability():
                self.draw_head = self.draw_head_navigable
            elif self._head_end.get_navigability() == False:
                self.draw_head = self.draw_head_none
            else:
                self.draw_head = self.draw_head_undefined

            if head_subject.aggregation == intern('composite'):
                self.draw_tail = self.draw_tail_composite
            elif head_subject.aggregation == intern('shared'):
                self.draw_tail = self.draw_tail_shared
            elif self._tail_end.get_navigability():
                self.draw_tail = self.draw_tail_navigable
            elif self._tail_end.get_navigability() == False:
                self.draw_tail = self.draw_tail_none
            else:
                self.draw_tail = self.draw_tail_undefined

            if self._show_direction:
                m = len(self._handles) / 2
                h0, h1 = self._handles[m - 1], self._handles[m]
                self._dir_angle = atan2(h1.y - h0.y, h1.x - h0.x)
                self._dir_pos = (h0.x + h1.x) / 2, (h0.y + h1.y) / 2
                if self.tail_end.subject is self.subject.memberEnd[0]:
                    self._dir_angle += pi
        else:
            self.draw_head = self.draw_head_undefined
            self.draw_tail = self.draw_tail_undefined

        # update relationship after self.set calls to avoid circural updates
        NamedLine.post_update(self, context)

        # Calculate alignment of the head name and multiplicity
        self._head_end.post_update(context, handles[0].pos,
                                     handles[1].pos)

        # Calculate alignment of the tail name and multiplicity
        self._tail_end.post_update(context, handles[-1].pos,
                                     handles[-2].pos)
        

    def point(self, x, y):
        """Returns the distance from the Association to the (mouse) cursor.
        """
        return min(super(AssociationItem, self).point(x, y),
                   self._head_end.point(x, y),
                   self._tail_end.point(x, y))

    def draw_head_none(self, context):
        """Draw an 'x' on the line end, indicating no traversing.
        """
        cr = context.cairo
        cr.move_to(6, -4)
        cr.rel_line_to(8, 8)
        cr.rel_move_to(0, -8)
        cr.rel_line_to(-8, 8)
        cr.stroke()
        cr.move_to(0, 0)

    def draw_tail_none(self, context):
        """Draw an 'x' on the line end, indicating no traversing.
        """
        cr = context.cairo
        cr.line_to(0, 0)
        cr.move_to(6, -4)
        cr.rel_line_to(8, 8)
        cr.rel_move_to(0, -8)
        cr.rel_line_to(-8, 8)
        cr.stroke()

    def _draw_diamond(self, cr):
        cr.move_to(20, 0)
        cr.line_to(10, -6)
        cr.line_to(0, 0)
        cr.line_to(10, 6)
        #cr.line_to(20, 0)
        cr.close_path()

    def draw_head_composite(self, context):
        """Draw a closed diamond on the line end.
        """
        cr = context.cairo
        self._draw_diamond(cr)
        context.cairo.fill_preserve()
        cr.stroke()
        cr.move_to(20, 0)

    def draw_tail_composite(self, context):
        """Draw a closed diamond on the line end.
        """
        cr = context.cairo
        cr.line_to(20, 0)
        cr.stroke()
        self._draw_diamond(cr)
        cr.fill_preserve()
        cr.stroke()

    def draw_head_shared(self, context):
        """Draw an open diamond on the line end.
        """
        cr = context.cairo
        self._draw_diamond(cr)
        cr.move_to(20, 0)

    def draw_tail_shared(self, context):
        """Draw an open diamond on the line end.
        """
        cr = context.cairo
        cr.line_to(20, 0)
        cr.stroke()
        self._draw_diamond(cr)
        cr.stroke()

    def draw_head_navigable(self, context):
        """Draw a normal arrow.
        """
        cr = context.cairo
        cr.move_to(15, -6)
        cr.line_to(0, 0)
        cr.line_to(15, 6)
        cr.stroke()
        cr.move_to(0, 0)

    def draw_tail_navigable(self, context):
        """Draw a normal arrow.
        """
        cr = context.cairo
        cr.line_to(0, 0)
        cr.stroke()
        cr.move_to(15, -6)
        cr.line_to(0, 0)
        cr.line_to(15, 6)

    def draw_head_undefined(self, context):
        """Draw nothing. undefined.
        """
        context.cairo.move_to(0, 0)

    def draw_tail_undefined(self, context):
        """Draw nothing. undefined.
        """
        context.cairo.line_to(0, 0)

    def draw(self, context):
        cr = context.cairo
        super(AssociationItem, self).draw(context)
        self._head_end.draw(context)
        self._tail_end.draw(context)
        if self._show_direction:
            cr.save()
            try:
                cr.translate(*self._dir_pos)
                cr.rotate(self._dir_angle)
                cr.move_to(0, 0)
                cr.line_to(6, 5)
                cr.line_to(0, 10)
                cr.fill()
            finally:
                cr.restore()


    def item_at(self, x, y):
        if distance_point_point_fast(self._handles[0].pos, (x, y)) < 10:
            return self._head_end
        elif distance_point_point_fast(self._handles[-1].pos, (x, y)) < 10:
            return self._tail_end
        return self
        
        
class AssociationEnd(SubjectSupport):
    """
    An association end represents one end of an association. An association
    has two ends. An association end has two labels: one for the name and
    one for the multiplicity (and maybe one for tagged values in the future).

    An AsociationEnd has no ID, hence it will not be stored, but it will be
    recreated by the owning Association.
    
    TODO:
    - add on_point() and let it return min(distance(_name), distance(_mult)) or
      the first 20-30 units of the line, for association end popup menu.
    """

    head_popup_menu = (
        'Head_unknownNavigation',
        'Head_isNotNavigable',
        'Head_isNavigable',
        'separator',
        'Head_AggregationNone',
        'Head_AggregationShared',
        'Head_AggregationComposite'
    )

    tail_popup_menu = (
        'Tail_unknownNavigation',
        'Tail_isNotNavigable',
        'Tail_isNavigable',
        'separator',
        'Tail_AggregationNone',
        'Tail_AggregationShared',
        'Tail_AggregationComposite'
    )

    def __init__(self, owner, id=None, end=None):
        SubjectSupport.__init__(self)
        self._owner = owner
        self._end = end
        
        # Rendered text for name and multiplicity
        self._name = None
        self._mult = None

        self._name_bounds = Rectangle()
        self._mult_bounds = Rectangle()

        self._subject = None

    def _set_subject(self, value):
        self._subject = value
        class pspec:
            name = 'subject'
        self.on_subject_notify(pspec)

    def _del_subject(self):
        self._subject = None
        class pspec:
            name = 'subject'
        self.on_subject_notify(pspec)

    subject = property(lambda s: s._subject, _set_subject, _del_subject)

    def get_popup_menu(self):
        if self.subject:
            if self._end == 'head':
                return self.head_popup_menu
            else:
                return self.tail_popup_menu
        elif self.parent:
            return self.parent.get_popup_menu()

    def request_update(self):
        self._owner.request_update()

    def set_text(self):
        """Set the text on the association end.
        """
        if self.subject:
            try:
                n, m = self.subject.render()
            except ValueError:
                # need more than 0 values to unpack: property was rendered as
                # attribute while in a UNDO action for example.
                pass
            else:
                self._name = n
                self._mult = m
                self.request_update()

    def get_navigability(self):
        """
        Check navigability of the AssociationEnd. If property is owned by
        class via ownedAttribute, then it is navigable. If property is
        owned by association by owndedEnd, then it is not navigable.
        Otherwise the navigability is unknown.

        Returned navigability values:
            - None  - unknown
            - False - not navigable
            - True  - navigable
        """
        navigability = None # unknown navigability as default
        subject = self.subject

        if subject and subject.opposite:
            opposite = subject.opposite
            if isinstance(opposite.type, UML.Interface):
                type = subject.interface_
            else: # isinstance(opposite.type, UML.Class):
                type = subject.class_

            if type and subject in type.ownedAttribute:
                navigability = True
            elif subject.association and subject in subject.association.ownedEnd:
                navigability = False

        return navigability
                

    def _set_navigability(self, navigable):
        """
        Change the AssociationEnd's navigability.

        A warning is issued if the subject or opposite property is missing.
        """
        subject = self.subject
        if subject and subject.opposite:
            opposite = subject.opposite

            #
            # Remove any navigability info, so unknown navigability state
            # is the default.
            #

            # if navigable
            if isinstance(opposite.type, UML.Class):
                if subject.class_:
                    del subject.class_
            elif isinstance(opposite.type, UML.Interface):
                if subject.interface_:
                    del subject.interface_
            else:
                assert 0, 'Should never be reached'

            # if not navigable
            if subject.owningAssociation:
                del subject.owningAssociation


            #
            # Set navigability.
            #
            if navigable:
                if isinstance(opposite.type, UML.Class):
                    subject.class_ = opposite.type
                elif isinstance(opposite.type, UML.Interface):
                    subject.interface_ = opposite.type
                else:
                    assert 0, 'Should never be reached'
            elif navigable == False:
                subject.owningAssociation = subject.association

            # else navigability is unknown

        else:
            log.warning('AssociationEnd.set_navigable: %s missing' % \
                        (subject and 'subject' or 'opposite Property'))

    navigability = property(get_navigability, _set_navigability)

    def point_name(self, x, y):
        p = (x, y)
        drp = distance_rectangle_point
        return drp(self._name_bounds, p)

    def point_mult(self, x, y):
        p = (x, y)
        drp = distance_rectangle_point
        return drp(self._mult_bounds, p)

    def point(self, x, y):
        return min(self.point_name(x, y), self.point_mult(x, y))

    def get_name(self):
        return self._name

    def get_mult(self):
        return self._mult

    def post_update(self, context, p1, p2):
        """
        Update label placement for association's name and
        multiplicity label. p1 is the line end and p2 is the last
        but one point of the line.
        """
        cr = context.cairo
        ofs = 5

        name_dx = 0.0
        name_dy = 0.0
        mult_dx = 0.0
        mult_dy = 0.0

        dx = float(p2[0]) - float(p1[0])
        dy = float(p2[1]) - float(p1[1])
        
        name_w, name_h = map(max, text_extents(cr, self._name, multiline=True), (10, 10))
        mult_w, mult_h = map(max, text_extents(cr, self._mult, multiline=True), (10, 10))

        if dy == 0:
            rc = 1000.0 # quite a lot...
        else:
            rc = dx / dy
        abs_rc = abs(rc)
        h = dx > 0 # right side of the box
        v = dy > 0 # bottom side

        if abs_rc > 6:
            # horizontal line
            if h:
                name_dx = ofs
                name_dy = -ofs - name_h
                mult_dx = ofs
                mult_dy = ofs
            else:
                name_dx = -ofs - name_w
                name_dy = -ofs - name_h
                mult_dx = -ofs - mult_w
                mult_dy = ofs
        elif 0 <= abs_rc <= 0.2:
            # vertical line
            if v:
                name_dx = -ofs - name_w
                name_dy = ofs
                mult_dx = ofs
                mult_dy = ofs
            else:
                name_dx = -ofs - name_w
                name_dy = -ofs - name_h
                mult_dx = ofs
                mult_dy = -ofs - mult_h
        else:
            # Should both items be placed on the same side of the line?
            r = abs_rc < 1.0

            # Find out alignment of text (depends on the direction of the line)
            align_left = (h and not r) or (r and not h)
            align_bottom = (v and not r) or (r and not v)
            if align_left:
                name_dx = ofs
                mult_dx = ofs
            else:
                name_dx = -ofs - name_w
                mult_dx = -ofs - mult_w
            if align_bottom:
                name_dy = -ofs - name_h
                mult_dy = -ofs - name_h - mult_h
            else:
                name_dy = ofs 
                mult_dy = ofs + mult_h

        self._name_bounds = Rectangle(p1[0] + name_dx,
                                      p1[1] + name_dy,
                                      width=name_w,
                                      height=name_h)

        self._mult_bounds = Rectangle(p1[0] + mult_dx,
                                      p1[1] + mult_dy,
                                      width=mult_w,
                                      height=mult_h)

    def on_subject_notify(self, pspec, notifiers=()):
        SubjectSupport.on_subject_notify(self, pspec,
                        notifiers + ('aggregation', 'visibility',
                        'name', 'lowerValue.value',
                        'upperValue.value', 'taggedValue',
                        'owningAssociation', 'class_', 'interface_'))
        #print 'w/ assoc', self.subject and self.subject.association
        self.set_text()
        self.request_update()
        
    def on_subject_notify__aggregation(self, subject, pspec):
        self.request_update()

    def on_subject_notify__name(self, subject, pspec):
        self.set_text()

    def on_subject_notify__visibility(self, subject, pspec):
        self.set_text()

    def on_subject_notify__lowerValue_value(self, lower_value, pspec):
        log.debug('New value for lowerValue.value: %s' % lower_value and lower_value.value)
        self.set_text()
        self.request_update()

    def on_subject_notify__upperValue_value(self, upper_value, pspec):
        #log.debug('New value for upperValue.value: %s' %  upper_value and upper_value.value)
        self.set_text()
        self.request_update()

    def on_subject_notify__taggedValue(self, tagged_value, pspec):
        self.set_text()
        self.request_update()

    def on_subject_notify__owningAssociation(self, upper_value, pspec):
        self.request_update()

    def on_subject_notify__class_(self, upper_value, pspec):
        self.request_update()

    def on_subject_notify__interface_(self, upper_value, pspec):
        self.request_update()

    def point(self, x, y):
        """Given a point (x, y) return the distance to the canvas item.
        """
        p = (x, y)
        drp = distance_rectangle_point
        d1 = drp(self._name_bounds, p)
        d2 = drp(self._mult_bounds, p)
#        try:
#            d3 = geometry.distance_point_point(self._point1, p)
#            d4, dummy = distance_line_point(self._point1, self._point2, p, 1.0, 0) #diacanvas.shape.CAP_ROUND)
#            if d3 < 15 and d4 < 5:
#                d3 = 0.0
#        except Exception, e:
#            log.error("Could not determine distance", e)
        d3 = 1000.0
        return min(d1, d2, d3)

    def draw(self, context):
        """Draw name and multiplicity of the line end.
        """
        if not self.subject:
            return

        cr = context.cairo
        text_multiline(cr, self._name_bounds[0], self._name_bounds[1], self._name)
        text_multiline(cr, self._mult_bounds[0], self._mult_bounds[1], self._mult)
        cr.stroke()

        if context.hovered or context.focused or context.draw_all:
            cr.set_line_width(0.5)
            b = self._name_bounds
            cr.rectangle(b.x, b.y, b.width, b.height)
            cr.stroke()
            b = self._mult_bounds
            cr.rectangle(b.x, b.y, b.width, b.height)
            cr.stroke()
    

# vim:sw=4:et



"""
CommentLine -- A line that connects a comment to another model element.

"""

import gobject
from gaphor import UML

from diagramline import DiagramLine

class CommentLineItem(DiagramLine):

    def __init__(self, id=None):
        DiagramLine.__init__(self, id)

    def save (self, save_func):
        DiagramLine.save(self, save_func)
    
    def load (self, name, value):
        DiagramLine.load(self, name, value)

    def postload(self):
        DiagramLine.postload(self)

    def on_notify_comment_parent(self, comment, pspec):
        if not comment.parent and self.parent:
            self.parent.remove(self)
            
    def draw(self, context):
        context.cairo.set_dash((7.0, 5.0), 0)
        DiagramLine.draw(self, context)


# vim: sw=4:et:ai



"""
A GUI for the checkmodel plugin.
"""

import sys
import gobject
import gtk
from zope import interface, component
from gaphor.core import _, inject, action, build_action_group
from gaphor.interfaces import IService, IActionProvider
import checkmodel

PYELEMENT_COLUMN = 0
ELEMENT_COLUMN = 1
REASON_COLUMN = 2

class CheckModelWindow(object):

    interface.implements(IService, IActionProvider)

    element_factory = inject('element_factory')
    gui_manager = inject('gui_manager')

    menu_xml = """
      <ui>
        <menubar action="mainwindow">
          <menu action="tools">
            <menuitem action="tools-open-check-model" />
          </menu>
        </menubar>
      </ui>"""

    def __init__(self):
        # Override the report method
        checkmodel.report = self.on_report
        self.action_group = build_action_group(self)

    def init(self, app):
        pass

    def shutdown(self):
        pass

    @action(name='tools-open-check-model', label='Check UML model')
    def open(self):
        self.construct()
        self.run()

    def construct(self):
        model = gtk.ListStore(gobject.TYPE_PYOBJECT,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING)

        treeview = gtk.TreeView(model)
        treeview.connect('row-activated', self.on_row_activated)
        selection = treeview.get_selection()
        selection.set_mode('single')
        treeview.set_size_request(200, -1)

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.add(treeview)
        scrolled_window.show()

        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Element", cell, text=ELEMENT_COLUMN)
        treeview.append_column(column)

        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Reason", cell, text=REASON_COLUMN)
        treeview.append_column(column)
        treeview.show()

        #self._construct_window(name='checkmodel',
        #                       title='Check Model',
        #                       size=(400, 400),
        #                       contents=scrolled_window)
        self.model = model
        self.treeview = treeview

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.connect('destroy', self.on_destroy)
        self.window.set_title('Gaphor - Check Model')
        self.window.add(scrolled_window)
        self.window.set_size_request(400, 400)
        self.window.show()
        
        
    def run(self):
        # TODO: Let this run in a Thread(?)
        checkmodel.check_classes(self.element_factory)
        checkmodel.check_attributes(self.element_factory)
        checkmodel.check_associations(self.element_factory)

    def on_report(self, element, message):
        log.info('%s: %s' % (type(element).__name__, message))
        model = self.model
        iter = model.append()
        model.set_value(iter, PYELEMENT_COLUMN, element)
        model.set_value(iter, ELEMENT_COLUMN, type(element).__name__)
        model.set_value(iter, REASON_COLUMN, message)
        main = gobject.main_context_default()
        main.iteration(False)

    def on_row_activated(self, treeview, row, column):
        iter = self.model.get_iter(row)
        element = self.model.get_value(iter, PYELEMENT_COLUMN)
        print 'Looking for element', element
        if element.presentation:
            main_window = self.gui_manager.main_window
            presentation = element.presentation[0]
            try:
                diagram = presentation.canvas.diagram
            except AttributeError:
                presentation = element.namespace.presentation[0]
                diagram = presentation.canvas.diagram
            diagram_tab = main_window.show_diagram(diagram)
            diagram_tab.view.focused_item = presentation

    def on_destroy(self, window):
        self.window = None
        self.treeview = None

# vim:sw=4:et



"""
The main application window.
"""

import gobject, gtk

from zope import interface, component
from gaphor.interfaces import IActionProvider
from interfaces import IUIComponent

from gaphor import UML
from gaphor.core import _, inject, action, radio_action, build_action_group
from namespace import NamespaceModel, NamespaceView
from diagramtab import DiagramTab
from toolbox import Toolbox
from diagramtoolbox import TOOLBOX_ACTIONS
from propertyeditor import PropertyEditor
from toplevelwindow import ToplevelWindow


from interfaces import IDiagramSelectionChange
from gaphor.interfaces import IServiceEvent, IActionExecutedEvent
from event import DiagramSelectionChange
from gaphor.application import Application
from gaphor.services.filemanager import FileManagerStateChanged

class MainWindow(ToplevelWindow):
    """
    The main window for the application.
    It contains a Namespace-based tree view and a menu and a statusbar.
    """
    interface.implements(IActionProvider)

    properties = inject('properties')
    element_factory = inject('element_factory')
    action_manager = inject('action_manager')
    file_manager = inject('file_manager')

    title = 'Gaphor'
    size = property(lambda s: s.properties.get('ui.window-size', (760, 580)))
    menubar_path = '/mainwindow'
    toolbar_path = '/mainwindow-toolbar'

    menu_xml = """
      <ui>
        <menubar name="mainwindow">
          <menu action="file">
            <placeholder name="primary" />
            <separator />
            <menu action="file-export" />
            <menu action="file-import" />
            <separator />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
            <separator />
            <menuitem action="file-quit" />
          </menu>
          <menu action="edit">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="diagram">
            <menuitem action="tree-view-create-diagram" />
            <menuitem action="tree-view-delete-diagram" />
            <separator />
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="tools">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="window">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="help">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
        </menubar>
        <toolbar name='mainwindow-toolbar'>
        </toolbar>
        <toolbar action="tools">
        </toolbar>
        <popup action="namespace-popup">
          <menuitem action="tree-view-open" />
          <menuitem action="tree-view-rename" />
          <separator />
          <menuitem action="tree-view-create-diagram" />
          <menuitem action="tree-view-delete-diagram" />
          <separator />
          <menuitem action="tree-view-refresh" />
        </popup>
      </ui>
    """

    def __init__(self):
        ToplevelWindow.__init__(self)
        # Map tab contents to DiagramTab
        self.notebook_map = {}
        # Tree view:
        self._tree_view = None 

        self.action_group = build_action_group(self)
        for name, label in (('file', '_File'),
                             ('file-export', '_Export'),
                             ('file-import', '_Import'),
                             ('edit', '_Edit'),
                             ('diagram', '_Diagram'),
                             ('tools', '_Tools'),
                             ('window', '_Window'),
                             ('help', '_Help')):
            a = gtk.Action(name, label, None, None)
            a.set_property('hide-if-empty', False)
            self.action_group.add_action(a)
        self._tab_ui_settings = None

    tree_model = property(lambda s: s.tree_view.get_model())

    tree_view = property(lambda s: s._tree_view)

    def get_filename(self):
        """
        Return the file name of the currently opened model.
        """
        return self.file_manager.filename

    def get_current_diagram_tab(self):
        """
        Get the currently opened and viewed DiagramTab, shown on the right
        side of the main window.
        See also: get_current_diagram(), get_current_diagram_view().
        """
        return self.get_current_tab()

    def get_current_diagram(self):
        """
        Return the Diagram associated with the viewed DiagramTab.
        See also: get_current_diagram_tab(), get_current_diagram_view().
        """
        tab = self.get_current_diagram_tab()
        return tab and tab.get_diagram()

    def get_current_diagram_view(self):
        """
        Return the DiagramView associated with the viewed DiagramTab.
        See also: get_current_diagram_tab(), get_current_diagram().
        """
        tab = self.get_current_diagram_tab()
        return tab and tab.get_view()

    def ask_to_close(self):
        """
        Ask user to close window.
        """
        dialog = gtk.MessageDialog(self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
            _("Quit Gaphor?"))
        answer = dialog.run()
        dialog.destroy()
        return answer == gtk.RESPONSE_YES

    def show_diagram(self, diagram):
        """
        Show a Diagram element in a new tab.
        If a tab is already open, show that one instead.
        """
        # Try to find an existing window/tab and let it get focus:
        for tab in self.get_tabs():
            if tab.get_diagram() is diagram:
                self.set_current_page(tab)
                return tab

        tab = DiagramTab(self)
        tab.set_diagram(diagram)
        widget = tab.construct()
        self.add_tab(tab, widget, tab.title)
        self.set_current_page(tab)

        return tab

    def ui_component(self):
        """
        Create the widgets that make up the main window.
        """
        model = NamespaceModel(self.element_factory)
        view = NamespaceView(model, self.element_factory)
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.add(view)
        view.show()
        
        view.connect_after('event-after', self._on_view_event)
        view.connect('row-activated', self._on_view_row_activated)
        view.connect_after('cursor-changed', self._on_view_cursor_changed)

        vbox = gtk.VBox()
        vbox.pack_start(scrolled_window, expand=True)
        scrolled_window.show()

        paned = gtk.HPaned()
        paned.set_property('position', 160)
        paned.pack1(vbox)
        vbox.show()
        
        notebook = gtk.Notebook()
        notebook.set_scrollable(True)
        notebook.set_show_border(False)

        notebook.connect_after('switch-page', self._on_notebook_switch_page)
        notebook.connect_after('page-removed', self._on_notebook_page_removed)

        self.property_editor = PropertyEditor()
        pe_notebook = self.property_editor.construct()
        pe_notebook.set_size_request(-1, 50)

        second_paned = gtk.VPaned()
        second_paned.set_property('position',
                                 int(self.properties.get('ui.property-editor-position', 600)))
        second_paned.pack1(notebook)
        notebook.show()
        second_paned.pack2(pe_notebook)
        pe_notebook.show()
        
        paned.pack2(second_paned)
        second_paned.show()
        paned.show()

        second_paned.connect('notify::position',
                            self._on_property_editor_notify_position)

        self.notebook = notebook
        self._tree_view = view
       
        vbox.set_border_width(3)

        toolbox = Toolbox(TOOLBOX_ACTIONS)
        vbox.pack_start(toolbox, expand=False)
        toolbox.show()

        self._toolbox = toolbox

        return paned

    def construct(self):
        super(MainWindow, self).construct()

        self.window.connect('delete-event', self._on_window_delete)

        # We want to store the window size, so it can be reloaded on startup
        self.window.set_property('allow-shrink', True)
        self.window.connect('size-allocate', self._on_window_size_allocate)
        self.window.connect('destroy', self._on_window_destroy)

        Application.register_handler(self._action_executed)

    def _update_toolbox(self, action_group):
        """
        Update the buttons in the toolbox. Each button should be connected
        by an action. Each button is assigned a special _action_name_
        attribute that can be used to fetch the action from the ui manager.
        """
        for button in self._toolbox.buttons:
            
            action_name = button.action_name
            action = action_group.get_action(action_name)
            if action:
                action.connect_proxy(button)

    # Notebook methods:

    def add_tab(self, tab_id, contents, label):
        """
        Create a new tab on the notebook with window as its contents.
        Returns: The page number of the tab.
        """
        self.notebook_map[contents] = tab_id
        #contents.connect('destroy', self._on_tab_destroy)
        l = gtk.Label(label)
        # Note: append_page() emits switch-page event
        self.notebook.append_page(contents, l)
        page_num = self.notebook.page_num(contents)
        #self.notebook.set_current_page(page_num)
        return page_num

    def get_current_tab(self):
        """
        Return the window (DiagramTab) that is currently visible on the
        notebook.
        """
        current = self.notebook.get_current_page()
        content = self.notebook.get_nth_page(current)
        return self.notebook_map.get(content)

    def set_current_page(self, tab):
        """
        Force a specific tab (DiagramTab) to the foreground.
        """
        for p, t in self.notebook_map.iteritems():
            if tab is t:
                num = self.notebook.page_num(p)
                self.notebook.set_current_page(num)
                return

    def set_tab_label(self, tab, label):
        for p, t in self.notebook_map.iteritems():
            if tab is t:
                l = gtk.Label(label)
                l.show()
                self.notebook.set_tab_label(p, l)

    def get_tabs(self):
        return self.notebook_map.values()

    def remove_tab(self, tab):
        """
        Remove the tab from the notebook. Tab is such a thing as
        a DiagramTab.
        """
        for p, t in self.notebook_map.iteritems():
            if tab is t:
                num = self.notebook.page_num(p)
                self.notebook.remove_page(num)
                del self.notebook_map[p]
                return

    def select_element(self, element):
        """
        Select an element from the Namespace view.
        The element is selected. After this an action may be executed,
        such as OpenModelElement, which will try to open the element (if it's
        a Diagram).
        """
        path = self.tree_model.path_from_element(element)
        # Expand the first row:
        self._tree_view.expand_row(path[:-1], False)
        selection = self._tree_view.get_selection()
        selection.select_path(path)
        self._on_view_cursor_changed(self._tree_view)

    # Signal callbacks:

    @component.adapter(FileManagerStateChanged)
    def _action_executed(self, event):
        # We're only interested in file operations
        print 'action:', event.service
        if event.service is self.file_manager:
            filename = self.file_manager.filename
            if self.window:
                if filename:
                    title = '%s - %s' % (self.title, filename)
                else:
                    title = self.title
                self.window.set_title(title)


    def _on_window_destroy(self, window):
        """
        Window is destroyed... Quit the application.
        """
        self._tree_view = None
        self.window = None
        if gobject.main_depth() > 0:
            gtk.main_quit()
        Application.unregister_handler(self._action_executed)

    def _on_tab_destroy(self, widget):
        tab = self.notebook_map[widget]
        assert isinstance(tab, DiagramTab)
        self.remove_tab(tab)

    def _on_window_delete(self, window = None, event = None):
        return not self.ask_to_close()

    def _on_view_event(self, view, event):
        """
        Show a popup menu if button3 was pressed on the TreeView.
        """
        # handle mouse button 3:
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            menu = self.ui_manager.get_widget('/namespace-popup')
            menu.popup(None, None, None, event.button, event.time)


    def _on_view_row_activated(self, view, path, column):
        """
        Double click on an element in the tree view.
        """
        self.action_manager.execute('tree-view-open')

    def _on_view_cursor_changed(self, view):
        """
        Another row is selected, execute a dummy action.
        """
        element = view.get_selected_element()
        self.action_group.get_action('tree-view-create-diagram').props.sensitive = isinstance(element, UML.Package)
        self.action_group.get_action('tree-view-open').props.sensitive = isinstance(element, UML.Diagram)

    def _insensivate_toolbox(self):
        for button in self._toolbox.buttons:
            button.set_property('sensitive', False)

    def _on_notebook_page_removed(self, notebook, tab, page_num):
        if self._tab_ui_settings:
            action_group, ui_id = self._tab_ui_settings
            self.ui_manager.remove_action_group(action_group)
            self.ui_manager.remove_ui(ui_id)
            self._insensivate_toolbox()
            self._tab_ui_settings = None
            if notebook.get_current_page() != -1:
                self._on_notebook_switch_page(notebook, None, notebook.get_current_page())

    def _on_notebook_switch_page(self, notebook, tab, page_num):
        """
        Another page (tab) is put on the front of the diagram notebook.
        A dummy action is executed.
        """
        if self._tab_ui_settings:
            action_group, ui_id = self._tab_ui_settings
            self.ui_manager.remove_action_group(action_group)
            self.ui_manager.remove_ui(ui_id)
            self._tab_ui_settings = None

        content = self.notebook.get_nth_page(page_num)
        tab = self.notebook_map.get(content)
        assert isinstance(tab, DiagramTab), str(tab)
        
        self.ui_manager.insert_action_group(tab.action_group, -1)
        ui_id = self.ui_manager.add_ui_from_string(tab.menu_xml)
        self._tab_ui_settings = tab.action_group, ui_id
        log.debug('Menus updated with %s, %d' % self._tab_ui_settings)
        self._update_toolbox(tab.toolbox.action_group)

        # Make sure everyone knows the selection has changed.
        component.handle(DiagramSelectionChange(tab.view, tab.view.focused_item, tab.view.selected_items))

    def _on_window_size_allocate(self, window, allocation):
        """
        Store the window size in a property.
        """
        self.properties.set('ui.window-size', (allocation.width, allocation.height))

    def _on_property_editor_notify_position(self, paned, arg):
        self.properties.set('ui.property-editor-position',
                     paned.get_position())

    # Actions:

    @action(name='file-quit', stock_id='gtk-quit')
    def quit(self):
        self.ask_to_close() and gtk.main_quit()

    @action(name='tree-view-open', label='_Open')
    def tree_view_open_selected(self):
        element = self._tree_view.get_selected_element()
        if isinstance(element, UML.Diagram):
            self.show_diagram(element)
        else:
            log.debug('No action defined for element %s' % type(element).__name__)

    @action(name='tree-view-rename', label=_('Rename'))
    def tree_view_rename_selected(self):
        view = self._tree_view
        element = view.get_selected_element()
        path = view.get_model().path_from_element(element)
        column = view.get_column(0)
        cell = column.get_cell_renderers()[1]
        cell.set_property('editable', 1)
        cell.set_property('text', element.name)
        view.set_cursor(path, column, True)
        cell.set_property('editable', 0)

    @action(name='tree-view-create-diagram', label=_('_New diagram'), stock_id='gaphor-diagram')
    def tree_view_create_diagram(self):
        element = self._tree_view.get_selected_element()
        diagram = self.element_factory.create(UML.Diagram)
        diagram.package = element

        diagram.name = '%s diagram' % element.name

        self.select_element(diagram)
        self.show_diagram(diagram)
        self.tree_view_rename_selected()

    @action(name='tree-view-delete-diagram', label=_('_Delete diagram'), stock_id='gtk-delete')
    def tree_view_delete_diagram(self):
        diagram = self._tree_view.get_selected_element()
        assert isinstance(diagram, UML.Diagram)
        m = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION,
                              gtk.BUTTONS_YES_NO,
                              'Do you really want to delete diagram %s?\n\n'
                              'This will possibly delete diagram items\n'
                              'that are not shown in other diagrams.\n'
                              'This operation is not undoable!' \
                              % (diagram.name or '<None>'))
        if (m.run() == gtk.RESPONSE_YES):
            diagram.unlink()
        m.destroy()

    @action(name='tree-view-refresh', label=_('_Refresh'))
    def tree_view_refresh(self):
        self._tree_model.refresh()


gtk.accel_map_add_filter('gaphor')


# vim:sw=4:et:ai



#!/usr/bin/env python
'''This application shows a nice tree view of the UML meta model.

Somehow, this peace of code seems to crash by X server. I thinnk it's a bug in
GTK+. ... but how do I isolate the bug?
'''

import sys
sys.path.append("..")

from gaphor.UML import *
import gtk
import gobject
import types


# to create a new GtkTreeModel from python, you must derive from
# GenericTreeModel.
class TreeModel(gtk.GenericTreeModel):
    ''' The node is defined by a "name" tuple. '''

    def __init__(self, klass):
	'''constructor for the model.  Make sure you call
	PyTreeModel.__init__'''
	self.iter = None # Value used to keep track of iterations.
	self.klass = klass;
	
	gtk.GenericTreeModel.__init__(self)

    def class_from_node(self, node):
        klass = self.klass
        for n in node:
	    attrdef = klass._attrdef[n]
	    klass = attrdef[1]
	return klass

    # the implementations for TreeModel methods are prefixed with on_
    def on_get_flags(self):
	'''returns the GtkTreeModelFlags for this particular type of model'''
	return 0

    def on_get_n_columns(self):
	'''returns the number of columns in the model'''
	return 1

    def on_get_column_type(self, index):
	'''returns the type of a column in the model'''
	return gobject.TYPE_STRING

    def on_get_path(self, node):
	'''returns the tree path (a tuple of indices at the various
	levels) for a particular node.'''
	print "on_get_path", node
	return node

    def on_get_iter(self, path):
        '''returns the node corresponding to the given path. The patch is a
	   tuple of values, like (0 1 1). We have to figure out a path that is
	   easy to use by on_get_value() and can also be easely extended by
	   on_iter_children() and chopped by on_iter_parent()'''
	def get_iter(klass, path):
	    item = klass._attrdef.items()[path[0]]
	    if len(path) == 1:
		return (item[0],)
	    else:
	        subklass = klass._attrdef.items()[path[0]][1][1]
		return (item[0],) + get_iter(subklass, path[1:])
	
        node = get_iter(self.klass, path)
	
	#print "on_get_iter", path, node
	return node

    def on_get_value(self, node, column):
	'''returns the value stored in a particular column for the node'''
	assert column == 0
	attr = node[-1]
	parent = self.class_from_node(node[:-1])
	attrdef = parent._attrdef[attr]
	#print "on_get_value", node, attrdef
	closing = ")"
	if attrdef[0] == Sequence:
	    closing = "*)"
	    
	if len(attrdef) == 3:
	    return attr + " (" + attrdef[1].__name__ + "::" + attrdef[2] + closing
	else:
	    return attr + " (" + attrdef[1].__name__ + closing

    def on_iter_next(self, node):
	'''returns the next node at this level of the tree'''
	attr = node[-1]
	parent = self.class_from_node(node[:-1])
	attrdef = parent._attrdef[attr]
	items = parent._attrdef.items()
	sentinel = 0
	for i in items:
	    if sentinel:
	        node = node[:-1] + (i[0],)
		#print "on_iter_next", node
		return node
	    elif i[0] == attr:
	        sentinel = 1 # Use next item
	#print "on_iter_next (None)"
        return None

    def on_iter_children(self, node):
	'''returns the first child of this node'''
	klass = self.class_from_node(node)
	if klass.__dict__.has_key("_attrdef"):
	    item = klass._attrdef.items()[0][0]
	    #print "on_iter_children", node, item
	    return node + (item,)
	else:
	    #print "on_iter_children (None)"
	    return None

    def on_iter_has_child(self, node):
	'''returns true if this node has children'''
	#return 0
	klass = self.class_from_node(node)
	#print "on_iter_has_child", node
	return klass.__dict__.has_key("_attrdef") and len(klass._attrdef) > 0

    def on_iter_n_children(self, node):
	'''returns the number of children of this node'''
	klass = self.class_from_node(node)
	if klass.__dict__.has_key("_attrdef"):
	    items = klass._attrdef.items()
	    #print "on_iter_n_children", len(items)
	    return len(items) - 1
	else:
	    #print "on_iter_n_children (None)"
	    return 0

    def on_iter_nth_child(self, node, n):
	'''returns the nth child of this node'''
	#print "on_iter_nth_child"
        if node == None:
	    #print 'result =', n, self.klass._attrdef.items()[0][0]
            return (self.klass._attrdef.items()[0][0],)
        klass = self.class_from_node(node)
	if klass.__dict__.has_key("_attrdef") and len(node) >= n:
	    item = klass._attrdef.items()[n][0]
	    #print "on_iter_nth_child", node, klass.__name__, item
	    #print 'result =', n, node, item
	    return node + (item,)
	else:
	    #print 'result =', n, None
	    return None

    def on_iter_parent(self, node):
	'''returns the parent of this node'''
	#print "on_iter_parent", node
	return node[:-1]

def main():
    args = sys.argv[1:]
    
    base_class = None
    try:
        base_class = eval(args[0])
    except NameError:
        print "Invalid class name (" + args[0] + ")"
        sys.exit(1)
    except IndexError:
	print "Use gbrowseUML.py <class>, now using class ModelElement"
	base_class = ModelElement

    window = gtk.Window()
    window.connect('destroy', lambda win: gtk.main_quit())
    window.set_title('gbrowseUML -- ' + base_class.__name__)
    window.set_default_size(250, 400)

    scrolled_window = gtk.ScrolledWindow()
    scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    window.add(scrolled_window)

    model = TreeModel(base_class)
    tree_view = gtk.TreeView(model)
    cell = gtk.CellRendererText()
    # the text in the column comes from column 0
    column = gtk.TreeViewColumn(base_class.__name__, cell, text=0)
    tree_view.append_column(column)

    scrolled_window.add(tree_view)
    window.show_all()

    gtk.main()

if __name__ == '__main__':
    main()
else:
    del main



# vim:sw=4:et
'''
Feature diagram item. Feature is a super class of both Attribute, Operations and
Methods.
'''

from gaphas.item import Item
from diagramitem import DiagramItem
from gaphor.diagram import DiagramItemMeta
from gaphas.util import text_extents, text_set_font, text_align
import font

class FeatureItem(DiagramItem):
    """
    FeatureItems are model elements who recide inside a ClassifierItem, such
    as methods and attributes. Those items can have comments attached, but only
    on the left and right side.
    Note that features can also be used inside objects.
    """

    def __init__(self, id=None):
        DiagramItem.__init__(self, id)
        self.width = 0
        self.height = 0
        self.text = ''
        # Fool unlink code:
        self.canvas = None


    # Ensure we call the right connect functions:
    connect = DiagramItem.connect
    disconnect = DiagramItem.disconnect
    notify = DiagramItem.notify

    def save(self, save_func):
#        for prop in ('affine',):
#            self.save_property(save_func, prop)
        DiagramItem.save(self, save_func)
        
    def postload(self):
        if self.subject:
            self._expression.set_text(self.subject.render())

    def get_size(self, update=False):
        """
        Return the size of the feature. If update == True the item is
        directly updated.
        """
        return self.width, self.height

    def get_text(self):
        return ''

    def update_size(self, text, context):
        cr = context.cairo
        self.width, self.height = text_extents(cr, text)

    def on_subject_notify(self, pspec, notifiers=()):
        DiagramItem.on_subject_notify(self, pspec, notifiers)
        #log.debug('setting text %s' % self.subject.render() or '')
        self.text = self.subject and self.subject.render() or ''

    def point(self, x, y):
        """
        """
        return distance_rectangle_point((0, 0, self.width, self.height), (x, y))


class AttributeItem(FeatureItem):

    def __init__(self, id=None):
        FeatureItem.__init__(self, id)
        self.need_sync_attributes = False

    def on_subject_notify(self, pspec, notifiers=()):
        FeatureItem.on_subject_notify(self, pspec, ('name',
                                                    'isDerived',
                                                    'visibility',
                                                    'lowerValue.value',
                                                    'upperValue.value',
                                                    'defaultValue.value',
                                                    'typeValue.value',
                                                    'taggedValue',
                                                    'association')
                                                    + notifiers)
        #self._expression.set_text(self.subject.render() or '')
        #self.request_update()

    def on_subject_notify__name(self, subject, pspec):
        #log.debug('setting text %s' % self.subject.render() or '')
        #self._expression.set_text(self.subject.render() or '')
        self.request_update()

    on_subject_notify__isDerived = on_subject_notify__name
    on_subject_notify__visibility = on_subject_notify__name
    on_subject_notify__lowerValue_value = on_subject_notify__name
    on_subject_notify__upperValue_value = on_subject_notify__name
    on_subject_notify__defaultValue_value = on_subject_notify__name
    on_subject_notify__typeValue_value = on_subject_notify__name
    on_subject_notify__taggedValue = on_subject_notify__name

    def on_subject_notify__association(self, subject, pspec):
        """
        Make sure we update the attribute compartment (in case
        the class_ property was defined before it is connected to
        an association.
        """
        #if self.parent:
        #    self.parent.sync_attributes()
        self.need_sync_attributes = True
        self.request_update()

    def pre_update(self, context):
        if self.need_sync_attributes and context.parent:
            context.parent.sync_attributes()
        self.need_sync_attributes = False
        self.update_size(self.subject.render(), context)
        #super(AttributeItem, self).pre_update(context)

    def draw(self, context):
        cr = context.cairo
        text_set_font(cr, font.FONT)
        text_align(cr, 0, 0, self.subject.render() or '', align_x=1, align_y=1)
        #cr.show_text(self.subject.render() or '')



# TODO: handle Parameter's

class OperationItem(FeatureItem):

    def __init__(self, id=None):
        FeatureItem.__init__(self, id)
        self.need_sync_operations = False

    def on_subject_notify(self, pspec, notifiers=()):
        FeatureItem.on_subject_notify(self, pspec,
                        ('name', 'visibility', 'isAbstract')
                        + notifiers)

        # TODO: Handle subject.returnResult[*] and subject.formalParameter[*]

    def postload(self):
        FeatureItem.postload(self)
        self.need_sync_operations = False

    def on_subject_notify__name(self, subject, pspec):
        #log.debug('setting text %s' % self.subject.render() or '')
        self.request_update()

    on_subject_notify__isAbstract = on_subject_notify__name
    on_subject_notify__visibility = on_subject_notify__name
    on_subject_notify__taggedValue = on_subject_notify__name

    def pre_update(self, context):
        if self.need_sync_operations and context.parent:
            context.parent.sync_operations()
        self.need_sync_operations = False
        self.update_size(self.subject.render(), context)
        #super(OperationItem, self).pre_update(context)

    def draw(self, context):
        cr = context.cairo
        text_set_font(cr, font.FONT)
        text_align(cr, 0, 0, self.subject.render() or '', align_x=1, align_y=1)
        #cr.show_text(self.subject.render() or '')


# vim:sw=4:et



#!/usr/bin/env python
# vim: sw=4:et

import sys
import gtk
from zope import interface
import gaphor
from gaphor.interfaces import IActionProvider
from gaphor.ui.interfaces import IUIComponent
from gaphor.action import action, build_action_group
from gaphor.misc.console import GTKInterpreterConsole
from toplevelwindow import ToplevelWindow


class ConsoleWindow(ToplevelWindow):
    
    interface.implements(IActionProvider)

    menu_xml = """
        <ui>
          <menubar name="mainwindow">
            <menu action="tools">
              <menuitem action="ConsoleWindow:open" />
            </menu>
          </menubar>
          <menubar name="consolewindow">
            <menu action="file">
              <menuitem action="ConsoleWindow:close" />
            </menu>
          </menubar>
        </ui>
        """

    title = 'Gaphor Console'
    size = (400, 400)
    menubar_path = '/consolewindow'
    toolbar_path = ''

    def __init__(self):
        self.action_group = build_action_group(self)
        self.window = None
        self.ui_manager = None # injected

    def ui_component(self):
        console = GTKInterpreterConsole()
        console.show()
        return console

    @action(name='ConsoleWindow:open', label='_Console')
    def open(self):
        if not self.window:
            self.construct()
        else:
            self.window.show_all()

    @action(name='ConsoleWindow:close', stock_id='gtk-close', accel='<Control><Shift>w')
    def close(self):
        self.window.destroy()
        self.window = None




"""
The Core module provides an entry point for Gaphor's core constructs.

An average module should only need to import this module.
"""

from gaphor.application import Application as _Application
from gaphor.transaction import Transaction, transactional
from gaphor.action import action, toggle_action, radio_action, build_action_group
from gaphor.i18n import _


class inject(object):
    """
    Simple descriptor for dependency injection.
    This is technically a wrapper around Application.get_service().

    Usage::

      class A(object):
        gui_manager = inject('gui_manager')
    """
    
    def __init__(self, name):
        self._name = name
        self._inj = None
        
    def __get__(self, obj, class_=None):
        """
        Resolve a dependency, but only if we're called from an object instance.
        """
        if not obj:
            return self
        if self._inj is None:
            self._inj = _Application.get_service(self._name)
        return self._inj


# vim:sw=4:et:ai



"""
Package diagram item.
"""

from gaphor import UML
from gaphor.diagram.nameditem import NamedItem

class PackageItem(NamedItem):

    __uml__ = UML.Package, UML.Profile
    __stereotype__ = {
        'profile': UML.Profile,
    }
    __style__ = {
        'min-size': (NamedItem.style.min_size[0], 70),
        'name-padding': (25, 10, 5, 10),
        'tab-x': 50,
        'tab-y': 20,
    }

    def draw(self, context):
        super(PackageItem, self).draw(context)

        cr = context.cairo
        o = 0.0
        h = self.height
        w = self.width
        x = self.style.tab_x
        y = self.style.tab_y
        cr.move_to(x, y)
        cr.line_to(x, o)
        cr.line_to(o, o)
        cr.line_to(o, h)
        cr.line_to(w, h)
        cr.line_to(w, y)
        cr.line_to(o, y)
        cr.stroke()


# vim:sw=4:et



"""
Base classes related to items, which represent UML classes deriving
from NamedElement.
"""

from gaphor.diagram.elementitem import ElementItem
from gaphor.diagram.style import get_min_size, ALIGN_CENTER, ALIGN_TOP
import gaphor.diagram.font as font

class NamedItem(ElementItem):
    __namedelement__ = True

    __style__ = {
        'min-size'    : (100, 50),
        'name-align'  : (ALIGN_CENTER, ALIGN_TOP),
        'name-padding': (5, 10, 5, 10),
        'name-outside': False,
    }

    def __init__(self, id=None):
        """
        Create named item.
        """
        ElementItem.__init__(self, id)

        # create (from ...) text to distinguish diagram items from
        # different namespace
        self._from = self.add_text('from',
                pattern='(from %s)',
                style={ 'text-align-group': 'stereotype' },
                visible=self.is_namespace_info_visible,
                font=font.FONT_SMALL)

        # size of stereotype, namespace and name text
        self._header_size = 0, 0


    def is_namespace_info_visible(self):
        """
        Display name space info when it is different, then diagram's or
        parent's namespace.
        """
        subject = self.subject
        canvas = self.canvas

        if not subject or not canvas:
            return False

        namespace = subject.namespace
        parent = canvas.get_parent(self)

        # if there is a parent (i.e. interaction)
        if parent and parent.subject \
                and parent.subject.namespace is not namespace:
            return False

        return self._from.text and namespace is not canvas.diagram.namespace


    def on_subject_notify(self, pspec, notifiers=()):
        #log.debug('Class.on_subject_notify(%s, %s)' % (pspec, notifiers))
        ElementItem.on_subject_notify(self, pspec,
                ('namespace', 'namespace.name') + notifiers)
        if self.subject:
            self.on_subject_notify__namespace(self.subject)
                                    

    def on_subject_notify__namespace(self, subject, pspec=None):
        """
        Add a line '(from ...)' to the class item if subject's namespace
        is not the same as the namespace of this diagram.
        """
        subject = self.subject
        if subject and subject.namespace:
            self._from.text = subject.namespace.name
        else:
            self._from.text = ''
        self.request_update()


    def on_subject_notify__namespace_name(self, subject, pspec=None):
        """
        Change the '(from ...)' line if the namespace's name changes.
        """
        self.on_subject_notify__namespace(subject, pspec)


    def pre_update(self, context):
        """
        Calculate minimal size and header size.
        """
        super(NamedItem, self).pre_update(context)

        style = self._name.style

        # we can determine minimal size and header size only
        # when name is aligned inside an item
        if not style.text_outside:
            # at this stage stereotype text group should be already updated
            assert 'stereotype' in self._text_groups_sizes

            nw, nh = self._text_groups_sizes['stereotype']
            self._header_size = get_min_size(nw, nh, self.style.name_padding)

            self.min_width = max(self.style.min_size[0], self._header_size[0])
            self.min_height = max(self.style.min_size[1], self._header_size[1])


# vim:sw=4:et:ai



# vim:sw=4:et:ai

import unittest
import gaphor.UML as UML

class TestActivityNodes(unittest.TestCase):

    def testObjectFlow(self):
        """
        Check if flow is object flow when connect to object node.
        """
#        f = self.createFlow()
#
#        ai = self.createActionItem()
#        on = self.createObjectNode()
#
#        self.connectNodes(ai, on, f)
#
#        self.assertEqual(f.subject.__class__, UML.ObjectFlow)




# vim:sw=4:et:
"""
Convert a model from Gaphor version 0.2 to Gaphor version 0.3.
"""
import env
import operator

import gaphor.parser as parser
import gaphor.storage as storage
import gaphor.UML as UML
from gaphor.misc.uniqueid import generate_id

def get_ref(elem, key):
    try:
        return elem.references[key]
    except KeyError:
        return elem.references[properties_conversion_map[key]]

def _convert_multiplicity(elements, elem, prop):
    spec = parser.element(generate_id(), 'LiteralSpecification')
    spec.values['value'] = elem.values['multiplicity']
    elem.references['lowerValue'] = [ spec.id ]
    elements[spec.id] = spec
    del elem.values['multiplicity']

def _convert_aggregation(elements, elem, prop):
    """Strip first three characters from aggregation type (e.g. ak_none ->none)
    """
    #print elem, elem[prop]
    elem.values['aggregation'] = elem.values['aggregation'][3:]

def _convert_isNavigable(elements, elem, prop):
    if eval(elem['isNavigable']):
        # The property is navigable. This means that it is owned by the
        # class on the other end of the association.
        assoc_id = elem.references['association']
        assert len(assoc_id) == 1
        assoc = elements[assoc_id[0]]
        end_ids = get_ref(assoc, 'connection')
        other_end = elements[(end_ids[0] == elem.id) and end_ids[1] or end_ids[0]]
        clazz = elements[get_ref(other_end, 'participant')[0]]
        # Create an class_ -- ownedAttribute relationship
        attr = clazz.references.get('ownedAttribute')
        if attr is None:
            attr = clazz.references['ownedAttribute'] = []
        attr.append(elem.id)
        assert elem.references.get('class_') is None
        elem.references['class_'] = [clazz.id]
    else:
	elem.references['owningAssociation'] = elem['association']
    del elem.values['isNavigable']

def _convert_ownedElement(elements, elem, prop):
    for ref in elem.references['ownedElement']:
        if elements[ref].type == 'Diagram':
            own = 'ownedDiagram'
        elif elements[ref].type == 'Package':
            own = 'nestedPackage'
        else:
            own = 'ownedClassifier'
        l = elem.references.get(own)
        if l is None:
            l = elem.references[own] = []
        l.append(ref)

    del elem.references['ownedElement']

def remove (elements, elem, prop):
    del elem.references[prop]

# Property names are unique (unique enough anyway)
properties_conversion_map = {
    'comment': 'ownedComment',
    'namespace': 'package',
    # Association
    'connection': 'memberEnd',
    # AssociationEnd
    'aggregation': _convert_aggregation,
    'isNavigable': _convert_isNavigable,
    'multiplicity': _convert_multiplicity,
    'participant': 'type',
    # Attribute
    'Attribute.owner': 'class_',
    # Class
    'feature': 'ownedAttribute',
    'specialization': remove,
    'Class.association': remove,
    # Generalization
    'child': 'specific',
    'parent': 'general',
    # Package
    'ownedElement': _convert_ownedElement
}

classname_conversion_map = {
    'Model': 'Package',
    'Attribute': 'Property',
    'AssociationEnd': 'Property'
}

def convert_elements(elements):
    """Change the elements dictionary (as it is returned from
    gaphor.parser.parse()) and convert the entries so they can be loaded in
    the data model of the current version.
    """
    element_list = filter(lambda x: isinstance(x, parser.element), elements.values())
    # convert property names:
    for e in element_list:
        # First ensure that all AssociationEnd have isNavigable set
        # (This is the only attribute set to True by default):
        if e.type == 'AssociationEnd' and not e.values.get('isNavigable'):
            e.values['isNavigable'] = 'True'

	for p in e.values.keys() + e.references.keys():
	    func_or_str = properties_conversion_map.get('%s.%s' % (e.type, p)) \
                          or properties_conversion_map.get(p)
	    if func_or_str:
		if operator.isCallable(func_or_str):
		    func_or_str(elements, e, p)
		else:
		    if p in e.values.keys():
			e.values[func_or_str] = e.values[p]
			del e.values[p]
		    else:
			e.references[func_or_str] = e.references[p]
			del e.references[p]

    for e in element_list:
	if e.type in classname_conversion_map.keys():
	    e.type = classname_conversion_map[e.type]

    item_list = filter(lambda x: isinstance(x, parser.canvasitem), elements.values())
    for i in item_list:
        subject = i.references.get('subject')
        if subject:
            e = elements[subject[0]]
            p = e.references.get('presentation')
            if p is None:
                p = e.references['presentation'] = []
            p.append(i.id)

def convert_factory(factory):
    for a in factory.select(lambda e: isinstance(e, UML.Association)):
        # Link Association.package to the package which owns the diagram
        # the association is drawn in.
        p = a.presentation[0]
        a.package = p.canvas.diagram.namespace

        # If an association's end is not owned by a Class and is not
        # owned by the association, let it be owned by the association
        for p in a.memberEnd:
            if not p.class_ and not p.owningAssociation:
                p.owningAssociation = a

if __name__ == '__main__':
    import sys
    elements = parser.parse(sys.argv[1])
    convert_elements(elements)
    factory = UML.ElementFactory()
    storage._load(elements, factory)
    convert_factory(factory)
    storage.save('converted.gaphor', factory)




#!/usr/bin/env python

# Validate an Gaphor XML file using the Gaphor DTD

import libxml2 as xml
import sys

dtd = xml.parseDTD(None, '../doc/gaphor.dtd')

doc = xml.parseFile ('../gaphor/a.xml')

newdtd = doc.newDtd ('gaphor', None, '../doc/gaphor.dtd')

doc.addChild (newdtd)

doc.saveFormatFile ('-',  2)



# vim: sw=4
"""
This module contains a model element Diagram which is the abstract
representation of a UML diagram. Diagrams can be visualized and edited.
"""

__author__ = 'Arjan Molenaar'
__version__ = '$revision$'
__date__ = '$date$'

import gobject
import gaphas
from gaphor.misc import uniqueid
from uml2 import Namespace, PackageableElement

class DiagramCanvas(gaphas.Canvas):
    """
    Some additions to gaphas.Canvas class.
    Esp. load and save functionallity.
    """

    def __init__(self, diagram):
        super(DiagramCanvas, self).__init__()
        self._diagram = diagram
        self._block_updates = False

    diagram = property(lambda s: s._diagram)

    def _set_block_updates(self, block):
        self._block_updates = block
        if not block:
            self.update_now()

    block_updates = property(lambda s: s._block_updates, _set_block_updates)

    def update_now(self):
        if self._block_updates:
            #log.debug('Update blocked for canvas %s' % self)
            return
        super(DiagramCanvas, self).update_now()

    def save(self, save_func):
        #for prop in DiagramCanvas._savable_canvas_properties:
        #    save_func(prop, self.get_property(prop))
        #save_func('root_affine', self.root.get_property('affine'))
        # Save child items:
        for item in self.get_root_items():
            save_func(None, item)

    def postload(self):
        pass #self.block_updates = False

    def select(self, expression=lambda e: True):
        """
        Return a list of all canvas items that match expression.
        """
        return filter(expression, self.get_all_items())


class Diagram(Namespace, PackageableElement):
    """
    Diagrams may contain model elements and can be owned by a Package.
    """

    def __init__(self, id=None, factory=None):
        super(Diagram, self).__init__(id, factory)
        self.canvas = DiagramCanvas(self)

    def save(self, save_func):
        super(Diagram, self).save(save_func)
        save_func('canvas', self.canvas)

    def postload(self):
        super(Diagram, self).postload()
        self.canvas.postload()

    def create(self, type, parent=None, subject=None):
        """
        Create a new canvas item on the canvas. It is created with
        a unique ID and it is attached to the diagram's root item.
        """
        assert issubclass(type, gaphas.Item)
        obj = type(uniqueid.generate_id())
        if subject:
            obj.subject = subject
        self.canvas.add(obj, parent)
        return obj

    def unlink(self):
        # Make sure all canvas items are unlinked
        for item in self.canvas.get_all_items():
            try:
                item.unlink()
            except:
                pass

        super(Diagram, self).unlink()



from zope import component

import gtk

from gaphor.diagram.interfaces import IGroup
from gaphor.ui.diagramtools import PlacementTool
from gaphas.tool import ItemTool

# cursor to indicate grouping
IN_CURSOR = gtk.gdk.Cursor(gtk.gdk.DIAMOND_CROSS)

# cursor to indicate ungrouping
OUT_CURSOR = gtk.gdk.Cursor(gtk.gdk.SIZING)

class GroupPlacementTool(PlacementTool):
    """
    Try to group items when placing them on diagram.
    """

    def __init__(self, item_factory, after_handler=None, handle_index=-1):
        super(GroupPlacementTool, self).__init__(item_factory,
                after_handler,
                handle_index)

        self._parent = None


    def on_button_press(self, context, event):
        """
        If new item was placed onto diagram, then try to group it with
        parent using grouping adapter.
        """

        # first get parent
        self._parent = None
        if event.button == 1:
            context.ungrab()
            view = context.view
            self._parent = view.get_item_at_point(event.x, event.y)

        # now, place the new item
        placed = PlacementTool.on_button_press(self, context, event)

        # if there is a parent, then try to group item and parent
        if placed and self._parent:
            view = context.view
            item = view.focused_item
            
            adapter = component.queryMultiAdapter((self._parent, item), IGroup)
            if adapter and adapter.can_contain():
                adapter.group()

        return placed


    def on_motion_notify(self, context, event):
        """
        Change parent item to dropzone state if it can accept diagram item
        object to be created.
        """
        view = context.view

        if view.focused_item:
            view.unselect_item(view.focused_item)
            view.focused_item = None

        try:
            parent = view.get_item_at_point(event.x, event.y)
        except KeyError:
            # No bounding box yet.
            return

        if parent:
            adapter = component.queryMultiAdapter((parent, self._factory.item_class), IGroup)
            if adapter and adapter.pre_can_contain():
                view.dropzone_item = parent
                view.window.set_cursor(IN_CURSOR)
            else:
                view.dropzone_item = None
                view.window.set_cursor(None)
        else:
            view.dropzone_item = None
            view.window.set_cursor(None)


    def _create_item(self, context, x, y):
        """
        Create diagram item and place it within parent's boundaries.
        """
        # if no parent, then create with parent placement tool
        if not self._parent:
            return super(GroupPlacementTool, self)._create_item(context, x, y)

        item = self._factory(self._parent)
        
        view = context.view
        # get item position through parent world
        x, y = view.canvas.get_matrix_c2i(self._parent).transform_point(x, y)
        item.matrix.translate(x, y)

        view.dropzone_item = None
        view.window.set_cursor(None)

        return item


class GroupItemTool(ItemTool):
    """
    Group diagram item by dropping it on another item.

    Works only for one selected item, now.
    """

    def on_motion_notify(self, context, event):
        """
        Indicate possibility of grouping/ungrouping of selected item.
        """
        super(GroupItemTool, self).on_motion_notify(context, event)
        view = context.view

        if event.state & gtk.gdk.BUTTON_PRESS_MASK and len(view.selected_items) == 1:
            item = list(view.selected_items)[0]
            parent = view.canvas.get_parent(item)

            over = view.get_item_at_point(event.x, event.y, selected=False)
            assert over is not item

            if over is parent: # do nothing when item is over parent
                view.dropzone_item = None
                view.window.set_cursor(None)
                return

            if parent and not over:  # are we going to remove from parent?
                adapter = component.queryMultiAdapter((parent, item.__class__), IGroup)
                if adapter and adapter.pre_can_contain():
                    view.window.set_cursor(OUT_CURSOR)
                    view.dropzone_item = parent

            if over:       # are we going to add to parent?
                adapter = component.queryMultiAdapter((over, item.__class__), IGroup)
                if adapter and adapter.pre_can_contain():
                    view.dropzone_item = over
                    view.window.set_cursor(IN_CURSOR)


    def on_button_release(self, context, event):
        """
        Group item if it is dropped on parent's item. Ungroup item if it is
        moved out of its parent boundaries. Method also moves item from old
        parent to new one (regrouping).
        """
        super(GroupItemTool, self).on_button_release(context, event)
        view = context.view
        if event.button == 1 and len(view.selected_items) == 1:
            item = list(view.selected_items)[0]
            parent = view.canvas.get_parent(item)
            over = view.get_item_at_point(event.x, event.y, selected=False)
            assert over is not item

            if over is parent:
                return

            if parent: # remove from parent
                adapter = component.queryMultiAdapter((parent, item), IGroup)
                if adapter and adapter.can_contain():
                    adapter.ungroup()

                    canvas = view.canvas
                    canvas.reparent(item, None)

                    # reset item's position
                    px, py = canvas.get_matrix_c2i(parent).transform_point(0, 0)
                    item.matrix.translate(-px, -py)


            if over: # add to over (over becomes parent)
                adapter = component.queryMultiAdapter((over, item), IGroup)
                if adapter and adapter.can_contain():
                    adapter.group()

                    canvas = view.canvas
                    canvas.reparent(item, over)

                    # reset item's position
                    x, y = canvas.get_matrix_i2c(over).transform_point(0, 0)
                    item.matrix.translate(-x, -y)


        view.dropzone_item = None
        view.window.set_cursor(None)


# vim:sw=4:et:ai



import sys
sys.path.append('../build/lib')

import gaphor.UML as UML
import gaphor.storage as storage

file = sys.argv[1]
names = sys.argv[2:]
storage.load(file)
#clear()
select = GaphorResource(UML.ElementFactory).select

for c in select(lambda e: e.isKindOf(UML.Class) and e.name in names):
    print 'Class:', c.name
    if c.feature:
	print 'Attributes'
    for a in c.feature:
	print '\t%s' % a.name
    if c.association:
	print 'Associations'
    for a in c.association:
	for con in a.association.connection:
	    if con is not a and con.isNavigable:
		print '\t%s: %s[%s] %s' % (con.name, con.participant.name, con.multiplicity, a.aggregation)
    print




"""
Service dedicated to exporting diagrams to a varyity of file formats.
"""

import os
from zope import interface, component
from gaphor.core import _, inject, action, build_action_group
from gaphor.interfaces import IService, IActionProvider

import cairo
from gaphas.view import View
from gaphas.painter import ItemPainter
from gaphas.geometry import Rectangle

class DiagramExportManager(object):
    """
    Service for exporting diagrams as images (SVG, PNG, PDF).
    """

    interface.implements(IService, IActionProvider)

    gui_manager = inject('gui_manager')

    menu_xml = """
      <ui>
        <menubar action="mainwindow">
          <menu action="file">
            <menu action="file-export">
              <menuitem action="file-export-svg" />
              <menuitem action="file-export-png" />
              <menuitem action="file-export-pdf" />
              <separator />
            </menu>
          </menu>
        </menubar>
      </ui>
    """

    def __init__(self):
        self.action_group = build_action_group(self)

    def init(self, app):
        pass

    def shutdown(self):
        pass

    def update(self):
        tab = self.get_window().get_current_diagram_tab()
        self.sensitive = tab and True or False


    def save_dialog(self, diagram, title, ext):
        import gtk
        filename = (diagram.name or 'export') + ext

        filesel = gtk.FileChooserDialog(title = title,
            action = gtk.FILE_CHOOSER_ACTION_SAVE,
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        filesel.set_current_name(filename)

        save = False
        while True:
            response = filesel.run()
            filename = filesel.get_filename()

            if response == gtk.RESPONSE_OK:
                if os.path.exists(filename):
                    dialog = gtk.MessageDialog(filesel,
                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
                        _("The file %s already exists. Do you want to replace it with the file you are exporting to?") % filename)
                    answer = dialog.run()
                    dialog.destroy()
                    if answer == gtk.RESPONSE_YES:
                        save = True
                        break
                else:
                    save = True
                    break
            else:
                break

        filesel.destroy()

        if save and filename:
            return filename
        return None


    def save_svg(self, filename, canvas):
        log.debug('Exporting SVG image to: %s' % filename)
        view = View(canvas)
        view.painter = ItemPainter()

        # Update bounding boxes with a temporaly CairoContext
        # (used for stuff like calculating font metrics)
        tmpsurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        tmpcr = cairo.Context(tmpsurface)
        view.update_bounding_box(tmpcr)
        tmpcr.show_page()
        tmpsurface.flush()

        w, h = view.bounding_box.width, view.bounding_box.height
        surface = cairo.SVGSurface(filename, w, h)
        cr = cairo.Context(surface)
        view.matrix.translate(-view.bounding_box.x, -view.bounding_box.y)
        view.paint(cr)
        cr.show_page()
        surface.flush()
        surface.finish()


    def save_png(self, filename, canvas):
        log.debug('Exporting PNG image to: %s' % filename)
        view = View(canvas)
        view.painter = ItemPainter()

        # Update bounding boxes with a temporaly CairoContext
        # (used for stuff like calculating font metrics)
        tmpsurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        tmpcr = cairo.Context(tmpsurface)
        view.update_bounding_box(tmpcr)
        tmpcr.show_page()
        tmpsurface.flush()

        w, h = view.bounding_box.width, view.bounding_box.height
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(w+1), int(h+1))
        cr = cairo.Context(surface)
        view.matrix.translate(-view.bounding_box.x, -view.bounding_box.y)
        view.paint(cr)
        cr.show_page()
        surface.write_to_png(filename)

    def save_pdf(self, filename, canvas):
        log.debug('Exporting PDF image to: %s' % filename)
        view = View(canvas)
        view.painter = ItemPainter()

        # Update bounding boxes with a temporaly CairoContext
        # (used for stuff like calculating font metrics)
        tmpsurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        tmpcr = cairo.Context(tmpsurface)
        view.update_bounding_box(tmpcr)
        tmpcr.show_page()
        tmpsurface.flush()

        w, h = view.bounding_box.width, view.bounding_box.height
        surface = cairo.PDFSurface(filename, w, h)
        cr = cairo.Context(surface)
        view.matrix.translate(-view.bounding_box.x, -view.bounding_box.y)
        view.paint(cr)
        cr.show_page()
        surface.flush()
        surface.finish()

    @action(name='file-export-svg', label='Export to SVG',
            tooltip='Export the diagram to SVG')
    def save_svg_action(self):
        title = 'Export diagram to SVG'
        ext = '.svg'
        diagram = self.gui_manager.main_window.get_current_diagram()
        filename = self.save_dialog(diagram, title, ext)
        if filename:
            self.save_svg(filename, diagram.canvas)


    @action(name='file-export-png', label='Export to PNG',
            tooltip='Export the diagram to PNG')
    def save_png_action(self):
        title = 'Export diagram to PNG'
        ext = '.png'
        diagram = self.gui_manager.main_window.get_current_diagram()
        filename = self.save_dialog(diagram, title, ext)
        if filename:
            self.save_png(filename, diagram.canvas)


    @action(name='file-export-pdf', label='Export to PDF',
            tooltip='Export the diagram to PDF')
    def save_pdf_action(self):
        title = 'Export diagram to PDF'
        ext = '.pdf'
        diagram = self.gui_manager.main_window.get_current_diagram()
        filename = self.save_dialog(diagram, title, ext)
        if filename:
            self.save_pdf(filename, diagram.canvas)


# vim:sw=4:et:




"""
CommentItem diagram item
"""

from gaphor import UML
from elementitem import ElementItem
from gaphas.item import NW
from gaphas.util import text_multiline, text_extents


class CommentItem(ElementItem):

    __uml__ = UML.Comment

    EAR=15
    OFFSET=5

    def __init__(self, id=None):
        ElementItem.__init__(self, id)
        self.min_width = CommentItem.EAR + 2 * CommentItem.OFFSET
        self.height = 50
        self.width = 100

    def edit(self):
        #self.start_editing(self._body)
        pass

    def on_subject_notify(self, pspec):
        """
        See DiagramItem.on_subject_notify().
        """
        ElementItem.on_subject_notify(self, pspec, ('body',))
        self.request_update()

    def on_subject_notify__body(self, subject, pspec):
        self.request_update()

    # DiaCanvasItem callbacks:

    def pre_update(self, context):
        if not self.subject: return
        cr = context.cairo
        w, h = text_extents(cr, self.subject.body, multiline=True, padding=2)
        self.min_width = w + 10
        self.min_height = h + 10
        ElementItem.pre_update(self, context)

    def post_update(self, context):
        ElementItem.post_update(self, context)

    def draw(self, context):
        if not self.subject: return
        c = context.cairo
        # Width and height, adjusted for line width...
        ox = float(self._handles[NW].x)
        oy = float(self._handles[NW].y)
        w = self.width + ox
        h = self.height + oy
        ear = CommentItem.EAR
        c.move_to(w - ear, oy)
        line_to = c.line_to
        line_to(w - ear, oy + ear)
        line_to(w, oy + ear)
        line_to(w - ear, oy)
        line_to(ox, oy)
        line_to(ox, h)
        line_to(w, h)
        line_to(w, oy + ear)
        c.stroke()
	if self.subject.body:
	    # Do not print empty string, since cairo-win32 can't handle it.
            text_multiline(c, 5, 5, self.subject.body, padding=2)
	    #c.move_to(10, 15)
	    #c.show_text(self.subject.body)

# vim:sw=4



"""
Use case extension relationship.
"""

from gaphor import UML
from gaphor.diagram.include import IncludeItem


class ExtendItem(IncludeItem):
    """
    Use case extension relationship.
    """
    __uml__ = UML.Extend
    __stereotype__ = 'extend'



# vim:sw=4:et



import copy
import inspect
import types
import string
import random
import time
import os
import sys

# This module is taken from SMW

def __getHundredNanosecondsSinceGregorianReform__():
    """<<<The timestamp is a 60 bit value. For UUID version 1, this is
    represented by Coordinated Universal Time (UTC) as a count of
    100-nanosecond intervals since 00:00:00.00, 15 October 1582 (the
    date of Gregorian reform to the Christian calendar).>>>"""

    # Since 1.1.1970 (unix epoch time)
    # BUG probably platform-dependent
    ns = time.time() * 1000 * 1000 * 10

    # Amount of 100 nanoseconds between 15.Oct.1582 to 1.1.1970
    # Takes even leap years into account
    ns_greg = ((1970-1583) * 365 + 78 + 92) * 24 * 60 * 60 * 1000 * 1000 * 10

    # BUG can long conversion lose information?
    return long(ns) + ns_greg

#Place holder for a RNG
theRNG=None

# Our NIC's MAC
ethernet = ""
# Our clocksequence
clocksequence = None
# Latest UTC
latest_utc = 0

def __generateString__(value, n_nibbles):
    """Generates nibbles, and reverses the resulting string."""
    s = []
    for i in range(n_nibbles):
        digit = value & 15
        value //= 16
        s.append(string.hexdigits[digit])

    s.reverse()
    return string.join(s, "")
    

def __generateID__():
    """This function returns a string with a unique ID for XMI objects.
    It adheres to the DCE specification of generating globally
    unique id:s. Assume there are buglets in this implementation,
    the spec isn't _that_ clean on the matter.
    http://www.opengroup.org/onlinepubs/9629399/apdxa.htm
    """
    
    global theRNG
    global ethernet
    global clocksequence
    global latest_utc

    if not theRNG:
        theRNG = random.Random(time.time())
        # Get the ethernet address
        try:
            if sys.platform.find('linux') >= 0:
                #
                # RedHat Linux, Debian GNU/Linux, SunOS 2.6
                #
                f = os.popen("/sbin/ifconfig -a")
                while 1:
                    s = f.readline()
                    if not s:
                        raise Exception

                    i = s.find("HWaddr")
                    if i != -1:
                        ethernet = string.join(s[i:].split()[1].split(":"), "")
                        f.close()
                        break;
            elif sys.platform.find('darwin') >= 0 \
                 or sys.platform.find('sun') >= 0:
                #
                # Darwin (Mac OS X) and SunOS
                #
                f = os.popen("/sbin/ifconfig -a")
                while 1:
                    s = f.readline()
                    if not s:
                        raise Exception
                    i = s.find("ether")
                    if i != -1:
                        e = s[i:].split()[1].split(":")
                        for i in range(len(e)):
                            if len(e[i]) == 1:
                                e[i] = "0" + e[i]
                        ethernet = string.join(e, "")
                        f.close()
                        break
            elif sys.platform.find('win') >= 0:
                #
                # Windows 2000 (perhaps also NT and XP)
                #
                f = os.popen("ipconfig /all")
                while 1:
                    s = f.readline()
                    if not s:
                        raise Exception
                    
                    i = s.find("Physical Address")
                    if i != -1:
                        i = s.find(":")
                        if i != -1:
                            ethernet = string.join(s[i:].split()[1].split("-"), "")
                        f.close()
                        break
        except:
            # BUG: We randomize
            e = []
            for eth_i in range(12):
                e.append(string.hexdigits[theRNG.randrange(16)])
            ethernet = string.join(e, "")

        # Initialize clock sequence. 14 bits
        clocksequence = theRNG.randrange(16384)
        
    s = ""
    # the final DCE string is of the form
    # "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
    value = __getHundredNanosecondsSinceGregorianReform__()
    index = 0

    if value <= latest_utc:
        clocksequence += 1
    else:
        latest_utc = value

    # time_low
    s += __generateString__(value, 8)
    s += "-"
    
    # time_mid
    value >>= (4*8)
    s += __generateString__(value, 4)
    s += "-"
    value >>= (4*4)

    # time_hi_and_version
    value |= 1<<12 # UUID version 1
    s += __generateString__(value, 4)
    s += "-"

    value = clocksequence
    value |= 1<<15 # DCE variant
    s += __generateString__(value, 4)
    s += "-"

    s += ethernet

    return "DCE:%s" % s.upper()

def generate_id():
    return __generateID__()



# vim:sw=4:et
"""
Internationalization (i18n) support for Gaphor.

Here the _() function is defined that is used to translate text into
your native language.
"""

__all__ = [ '_' ]

import os

import gettext
import pkg_resources

# default locale dir = site-base/gaphor/data/locale/<lang>/LC_MESSAGES/gaphor.mo
localedir = os.path.join(pkg_resources.get_distribution('gaphor').location, \
                         'gaphor', 'data', 'locale')

try:
    catalog = gettext.Catalog('gaphor', localedir=localedir)
    #log.info('catalog = %s' % catalog.info())
    _ = catalog.gettext
except IOError, e:
    #log.error('Could not load locale catalog', e)
    def _(s): return s




from gaphor import UML

def pr_interface_deps(classifier, dep_type):
    """
    Return all interfaces, which are connected to a classifier with given
    dependency type.
    """
    return (dep.supplier[0] for dep in classifier.clientDependency \
        if dep.isKindOf(dep_type) and dep.supplier[0].isKindOf(UML.Interface))

def pr_rc_interface_deps(component, dep_type):
    """
    Return all interfaces, which are connected to realizing classifiers of
    specified component. Returned interfaces are connected to realizing
    classifiers with given dependency type.

    Generator of generators is returned. Do not forget to flat it later.
    """
    return (pr_interface_deps(r.realizingClassifier, dep_type) for r in component.realization)




# GConf wrapper
import gconf
from gconf import VALUE_BOOL, VALUE_INT, VALUE_STRING, VALUE_FLOAT
from types import StringType, IntType, FloatType

class Conf:
    def __init__ (self, appname, allowed={}):
        self._domain = '/apps/%s/' % appname
        self._allowed = allowed
        self._gconf_client = gconf.client_get_default ()
        
    def __getitem__ (self, attr):
        return self.get_value (attr)

    def __setitem__ (self, key, val):
        allowed = self._allowed
        if allowed.has_key (key):
            if not key in allowed[key]:
                good = ', '.join (allowed[key])
                raise 'ConfError', '%s must be one of: (%s)' % (key, good)
        self.set_value (key, val)

    def _get_type (self, key):
        KeyType = type (key)
        if KeyType == StringType:
            return 'string'
        elif KeyType == IntType:
            return 'int'
        elif KeyType == FloatType:
            return 'float'
        else:
            raise 'ConfError', 'unsupported type: %s' % str (KeyType)
        
    # Public functions

    def set_allowed (self, allowed):
        self._allowed = allowed
        
    def set_domain (self, domain):
        self._domain = domain
        
    def get_domain (self):
        return self._domain
    
    def get_gconf_client (self):
        return self._gconf_client
        
    def get_value (self, key):
        '''returns the value of key `key' ''' #'
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        value = self._gconf_client.get (self._domain + key)
        value_type = value.type
        if value_type == VALUE_BOOL:
            return value.get_bool ()
        elif value_type == VALUE_INT:
            return value.get_int ()
        elif value_type == VALUE_STRING:
            return value.get_string ()
        elif value_type == VALUE_FLOAT:
            return value.get_float ()
    
    def set_value (self, key, value):
        '''sets the value of key `key' to `value' '''
        value_type = self._get_type (value)
        
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        func = getattr (self._gconf_client, 'set_' + value_type)
        apply (func, (self._domain + key, value))
        
    def get_string (self, key):
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        return self._gconf_client.get_string (self._domain + key)
    
    def set_string (self, key, value):
        if type (value) != StringType:
            raise 'ConfError', 'value must be a string'
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        self._gconf_client.set_string (self._domain + key, value)
        
    def get_bool (self, key):
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        return self._gconf_client.get_bool (self._domain + key)
    
    def set_bool (self, key, value):
        if type (value) != IntType and \
           (key != 0 or key != 1):
            raise 'ConfError', 'value must be a boolean'
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        self._gconf_client.set_bool (self._domain + key, value)
        
    def get_int (self, key):
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        return self._gconf_client.get_int (self._domain + key)
    
    def set_int (self, key, value):
        if type (value) != IntType:
            raise 'ConfError', 'value must be an int'
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        self._gconf_client.set_int (self._domain + key, value)
        
    def get_float (self, key):
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        return self._gconf_client.get_float (self._domain + key)
    
    def set_float (self, key, value):
        if type (value) != FloatType:
            raise 'ConfError', 'value must be an float'
        
        #if '/' in key:
        #    raise 'ConfError', 'key must not contain /'
        
        self._gconf_client.set_float (self._domain + key, value)

def test():
    c = Conf ('test-gconf')
    c['foo'] = '1'
    c['bar'] = 2
    c['baz'] = 3.0

    print c['foo'], c['bar'], c['baz']
    
if __name__ == '__main__':
    test()



#!/usr/bin/env python
# vim:sw=4:et:
"""Report differences in two Gaphor models.

This can be called as:
    python compare.py model1.gaphor model2.gaphor

This file is part of Gaphor.
"""

from __future__ import generators

__all__ = ['Compare']

try:
    import env
except ImportError:
    pass

import gaphor.parser
import gaphor.storage
import gaphor.UML

class Compare(object):
    """This class makes it possible to compare two files.
    By default reports are printed to stdout in a diff-like syntax.
    """

    def __init__(self, filename1, filename2):
        self.filename1 = filename1
        self.filename2 = filename2

        self.elements1, self.factory1 = self.load(self.filename1)
        self.elements2, self.factory2 = self.load(self.filename2)

        self.show_id = True

    def out(self, msg):
        """Print a message generated by report().
        """
        print msg
    
    def report(self, factory, element, name=None, value=None, isref=False):
        """Report an element that has differences.
        The attribute show_id can be set to False to suppress element ids.
        A fancy diff message is send to method out(msg).
        """
        if factory is self.factory1:
            msg = '-'
        else:
            msg = '+'
        
        if isinstance(element, gaphor.parser.canvas):
            msg += ' <canvas>:'
        else:
            if self.show_id:
                msg += ' %s' % element.id
            n = element.get('name')
            if n:
                msg += ' (%s)' % n
            if self.show_id or n:
                msg += ':'

        msg += ' %s' % (isinstance(element, gaphor.parser.canvas) and 'Canvas' or element.type)

        if name:
            msg += '.%s' % name
            if value:
                if isref:
                    if self.show_id:
                        msg += ' = %s' % value
                    obj = factory.lookup(value)
                    if hasattr(obj, 'name'):
                        msg += ' (%s)' % obj.name
                else:
                    msg += ' = %s' % value

        self.out(msg)

    def load(self, filename):
        """Load the model file and create a factory.
        A tuple (elements, factory) is returned.
        """
        elements = gaphor.parser.parse(filename)
        factory = gaphor.UML.ElementFactory()
        try:
            gaphor.storage.load_elements(elements, factory)
        except Exception, e:
            self.out('! File %s could not be loaded completely.' % filename)
            self.out('! Trying to diff on parsed elements only.')
            self.out(e)
        return elements, factory

    def elements_in_both_files(self):
        """Generator function that returns tuples (element1, element2) of
        elements that exist in both files (they have the same id).
        """
        vals = []
        for key1, val1 in self.elements1.iteritems():
            val2 = self.elements2.get(key1)
            if val2:
                yield (val1, val2)

    def check_missing_elements(self):
        """Report elements that exist in one factory, but not in the other.
        """
        keys1 = self.elements1.keys()
        keys2 = self.elements2.keys()
        for key in keys1:
            if key not in keys2:
                self.report(self.factory1, self.elements1[key])

        for key in keys2:
            if key not in keys1:
                self.report(self.factory2, self.elements2[key])

    def check_missing_references(self, element1, element2):
        """Report references to other elements that are present in one
        element and not in the other one.
        """
        keys1 = element1.references.keys()
        keys2 = element2.references.keys()
        for key in keys1:
            if key not in keys2:
                self.report(self.factory1, element1, key)

        for key in keys2:
            if key not in keys1:
                self.report(self.factory2, element2, key)

    def check_differences_references(self, element1, element2):
        keys1 = element1.references.keys()
        keys2 = element2.references.keys()
        for key in keys1:
            if key in keys2:
                val1 = element1.references.get(key)
                val2 = element2.references.get(key)
                try:
                    for val in val1:
                        if val not in val2:
                            self.report(self.factory1, element1, key, val, True)

                    for val in val2:
                        if val not in val1:
                            self.report(self.factory2, element2, key, val, True)
                except TypeError:
                    if val1 != val2:
                        self.report(self.factory1, element1, key, val1, True)
                        self.report(self.factory2, element2, key, val2, True)

    def check_missing_values(self, element1, element2):
        keys1 = element1.values.keys()
        keys2 = element2.values.keys()
        for key in keys1:
            if key not in keys2:
                self.report(self.factory1, element1, key)

        for key in keys2:
            if key not in keys1:
                self.report(self.factory2, element2, key)

    def check_differences_values(self, element1, element2):
        keys1 = element1.values.keys()
        keys2 = element2.values.keys()
        for key in keys1:
            if key in keys2:
                val1 = element1.values.get(key)
                val2 = element2.values.get(key)
                if val1 != val2:
                    self.report(self.factory1, element1, key, val1)
                    self.report(self.factory2, element2, key, val2)


    def compare(self):
        """Start the comparison of the files provided to the constructor.
        """
        self.check_missing_elements()

        for element1, element2 in self.elements_in_both_files():
            self.check_missing_references(element1, element2)
            self.check_differences_references(element1, element2)
            self.check_missing_values(element1, element2)
            self.check_differences_values(element1, element2)

if __name__ == '__main__':
    import sys

    usage = "usage: %s [-v][-h|--help] old_model new_model" % sys.argv[0]
    files = []
    show_id = False

    # Parse command line arguments:
    for arg in sys.argv[1:]:
        if arg.startswith('-'):
            if arg == '-v':
                show_id = True
            elif arg in ('-h', '--help'):
                print usage
                sys.exit(0)
            else:
                print '%s: invalid option "%s".' % (sys.argv[0], arg)
                print usage
                sys.exit(1)
        else:
            files.append(arg)

    if len(files) != 2:
        print usage
        sys.exit(1)

    c = Compare(files[0], files[1])
    c.show_id = show_id
    c.compare()





import gtk
import unittest

from gaphor.application import Application
from gaphor.ui.diagramtab import DiagramTab
from gaphor.ui.diagramtoolbox import DiagramToolbox
from gaphor import UML

class DiagramToolboxTestCase(unittest.TestCase):

    def setUp(self):
        Application.init(services=['element_factory', 'properties'])
        diagram = UML.Diagram()
        tab = DiagramTab(None)
        tab.diagram = diagram
        tab.construct()
        self.tab = tab

    def tearDown(self):
        Application.shutdown()

    def test_standalone_construct_with_diagram(self):
        pass # is setUp()

    def _test_placement_action(self, action):
        self.tab.toolbox.action_group.get_action(action).activate()
        assert self.tab.view.tool
        # Ensure the factory is working
        self.tab.view.tool._factory()

    def test_placement_pointer(self):
        self.tab.toolbox.action_group.get_action('toolbox-pointer').activate()

    def test_placement_comment(self):
        self._test_placement_action('toolbox-comment')

    def test_placement_comment_line(self):
        self._test_placement_action('toolbox-comment-line')

    # Classes:

    def test_placement_class(self):
        self._test_placement_action('toolbox-class')

    def test_placement_interface(self):
        self._test_placement_action('toolbox-interface')

    def test_placement_package(self):
        self._test_placement_action('toolbox-package')

    def test_placement_association(self):
        self._test_placement_action('toolbox-association')

    def test_placement_dependency(self):
        self._test_placement_action('toolbox-dependency')

    def test_placement_generalization(self):
        self._test_placement_action('toolbox-generalization')

    def test_placement_implementation(self):
        self._test_placement_action('toolbox-implementation')

    # Components:
    
    def test_placement_component(self):
        self._test_placement_action('toolbox-component')

    def test_placement_node(self):
        self._test_placement_action('toolbox-node')

    def test_placement_artifact(self):
        self._test_placement_action('toolbox-artifact')

    # Actions:

    def test_placement_action(self):
        self._test_placement_action('toolbox-action')

    def test_placement_initial_node(self):
        self._test_placement_action('toolbox-initial-node')

    def test_placement_activity_final_node(self):
        self._test_placement_action('toolbox-activity-final-node')

    def test_placement_flow_final_node(self):
        self._test_placement_action('toolbox-flow-final-node')

    def test_placement_decision_node(self):
        self._test_placement_action('toolbox-decision-node')

    def test_placement_fork_node(self):
        self._test_placement_action('toolbox-fork-node')

    def test_placement_object_node(self):
        self._test_placement_action('toolbox-object-node')

    def test_placement_flow(self):
        self._test_placement_action('toolbox-flow')

    # Use cases:

    def test_usecase(self):
        self._test_placement_action('toolbox-usecase')

    def test_actor(self):
        self._test_placement_action('toolbox-actor')

    def test_usecase_association(self):
        self._test_placement_action('toolbox-usecase-association')

    def test_include(self):
        self._test_placement_action('toolbox-include')

    def test_extend(self):
        self._test_placement_action('toolbox-extend')

    # Profiles:

    def test_profile(self):
        self._test_placement_action('toolbox-profile')

    def test_metaclass(self):
        self._test_placement_action('toolbox-metaclass')

    def test_stereotype(self):
        self._test_placement_action('toolbox-stereotype')

    def test_extension(self):
        self._test_placement_action('toolbox-extension')

# vim:sw=4:et:ai




import pkg_resources
from zope import interface
from gaphor.interfaces import IService


class AdapterLoader(object):
    """
    Initiate adapters from the gaphor.adapters module.
    """

    interface.implements(IService)

    def init(self, app):
        self._app = app
        import gaphor.adapters.connectors
        reload(gaphor.adapters.connectors)
        import gaphor.adapters.editors
        reload(gaphor.adapters.editors)
        import gaphor.adapters.grouping
        reload(gaphor.adapters.grouping)
        import gaphor.adapters.propertypages
        reload(gaphor.adapters.propertypages)

    def shutdown(self):
        pass


# vim:sw=4:et:ai



"""
Trivial drawing aids (box, line, ellipse).
"""


from gaphas.item import Line as _Line
from gaphas.item import Element, NW
from gaphas.util import path_ellipse
from style import Style

class Line(_Line):

    __style__ = {
        'line-width': 2,
        'line-color': (0, 0, 0, 1),
    }

    def __init__(self, id=None):
        super(Line, self).__init__()
        self.style = Style(Line.__style__)
        self._id = id
        self.fuzziness = 2
        self._handles[0].connectable = False
        self._handles[-1].connectable = False

    id = property(lambda self: self._id, doc='Id')

    def save (self, save_func):
        save_func('matrix', tuple(self.matrix))
        for prop in ('orthogonal', 'horizontal'):
            save_func(prop, getattr(self, prop))
        points = [ ]
        for h in self.handles():
            points.append(tuple(map(float, h.pos)))
        save_func('points', points)

    def load (self, name, value):
        if name == 'matrix':
            self.matrix = eval(value)
        elif name == 'points':
            points = eval(value)
            for x in xrange(len(points) - 2):
                self.split_segment(0)
            for i, p in enumerate(points):
                self.handles()[i].pos = p
        elif name == 'horizontal':
            self.horizontal = eval(value)
        elif name == 'orthogonal':
            self._load_orthogonal = eval(value)

    def postload(self):
        if hasattr(self, '_load_orthogonal'):
            self.orthogonal = self._load_orthogonal
            del self._load_orthogonal

    def draw(self, context):
        cr = context.cairo
        style = self.style
        cr.set_line_width(style.line_width)
        cr.set_source_rgba(*style.line_color)
        super(Line, self).draw(context)


class Box(Element):
    """
    A Box has 4 handles (for a start)::
     
    NW +---+ NE
    SW +---+ SE
    """

    __style__ = {
        'border-width': 2,
        'border-color': (0, 0, 0, 1),
        'fill-color': (1, 1, 1, 0),
    }

    def __init__(self, id=None):
        super(Box, self).__init__(10, 10)
        self.style = Style(Box.__style__)
        self._id = id

    id = property(lambda self: self._id, doc='Id')

    def save(self, save_func):
        save_func('matrix', tuple(self.matrix))
        save_func('width', self.width)
        save_func('height', self.height)

    def load(self, name, value):
        if name == 'matrix':
            self.matrix = eval(value)
        elif name == 'width':
            self.width = eval(value)
        elif name == 'height':
            self.height = eval(value)

    def postload(self):
        pass

    def draw(self, context):
        cr = context.cairo
        nw = self._handles[NW]
        style = self.style
        cr.rectangle(nw.x, nw.y, self.width, self.height)
        cr.set_source_rgba(*style.fill_color)
        cr.fill_preserve()
        cr.set_source_rgba(*style.border_color)
        cr.set_line_width(style.border_width)
        cr.stroke()


class Ellipse(Element):
    """
    """

    __style__ = {
        'border-width': 2,
        'border-color': (0, 0, 0, 1),
        'fill-color': (1, 1, 1, 0),
    }

    def __init__(self, id):
        super(Ellipse, self).__init__()
        self.style = Style(Ellipse.__style__)
        self._id = id

    id = property(lambda self: self._id, doc='Id')

    def save(self, save_func):
        save_func('matrix', tuple(self.matrix))
        save_func('width', self.width)
        save_func('height', self.height)

    def load(self, name, value):
        if name == 'matrix':
            self.matrix = eval(value)
        elif name == 'width':
            self.width = eval(value)
        elif name == 'height':
            self.height = eval(value)

    def postload(self):
        pass

    def draw(self, context):
        cr = context.cairo
        nw = self._handles[NW]
        style = self.style

        rx = self.width / 2.
        ry = self.height / 2.

        cr.move_to(self.width, ry)
        path_ellipse(cr, rx, ry, self.width, self.height)
        cr.set_source_rgba(*style.fill_color)
        cr.fill_preserve()
        cr.set_source_rgba(*style.border_color)
        cr.set_line_width(style.border_width)
        cr.stroke()


# vim:sw=4:et:ai



"""
Activity control nodes.
"""

import math

from gaphas.util import path_ellipse
from gaphas.state import observed, reversible_property
from gaphas.item import Handle, Item
from gaphas.constraint import EqualsConstraint, LessThanConstraint
from gaphas.geometry import distance_line_point

from gaphor import UML
from gaphor.core import inject
from gaphor.diagram.diagramitem import DiagramItem
from gaphor.diagram.nameditem import NamedItem
from gaphor.diagram.style import ALIGN_LEFT, ALIGN_CENTER, ALIGN_TOP, \
        ALIGN_RIGHT, ALIGN_BOTTOM
from gaphor.diagram.style import get_text_point


DEFAULT_JOIN_SPEC = 'and'


class ActivityNodeItem(NamedItem):
    """Basic class for simple activity nodes.
    Simple activity node is not resizable.
    """
    __style__   = {
        'name-outside': True,
        'name-padding': (2, 2, 2, 2),
    }

    def __init__(self, id=None):
        NamedItem.__init__(self, id)
        # Do not allow resizing of the node
        for h in self._handles:
            h.movable = False

        
class InitialNodeItem(ActivityNodeItem):
    """
    Representation of initial node. Initial node has name which is put near
    top-left side of node.
    """
    __uml__     = UML.InitialNode
    __style__   = {
        'min-size':   (20, 20),
        'name-align': (ALIGN_LEFT, ALIGN_TOP),
    }
    
    RADIUS = 10

    def draw(self, context):
        cr = context.cairo
        r = self.RADIUS
        d = r * 2
        path_ellipse(cr, r, r, d, d)
        cr.set_line_width(0.01)
        cr.fill()
        
        super(InitialNodeItem, self).draw(context)


class ActivityFinalNodeItem(ActivityNodeItem):
    """Representation of activity final node. Activity final node has name
    which is put near right-bottom side of node.
    """

    __uml__ = UML.ActivityFinalNode
    __style__   = {
        'min-size':   (30, 30),
        'name-align': (ALIGN_RIGHT, ALIGN_BOTTOM),
    }

    RADIUS_1 = 10
    RADIUS_2 = 15

    def draw(self, context):
        cr = context.cairo
        r = self.RADIUS_2 + 1
        d = self.RADIUS_1 * 2
        path_ellipse(cr, r, r, d, d)
        cr.set_line_width(0.01)
        cr.fill()

        d = r * 2
        path_ellipse(cr, r, r, d, d)
        cr.set_line_width(0.01)
        cr.set_line_width(2)
        cr.stroke()

        super(ActivityFinalNodeItem, self).draw(context)


class FlowFinalNodeItem(ActivityNodeItem):
    """
    Representation of flow final node. Flow final node has name which is
    put near right-bottom side of node.
    """

    __uml__ = UML.FlowFinalNode
    __style__   = {
        'min-size':   (20, 20),
        'name-align': (ALIGN_RIGHT, ALIGN_BOTTOM),
    }

    RADIUS = 10

    def draw(self, context):
        cr = context.cairo
        r = self.RADIUS
        d = r * 2
        path_ellipse(cr, r, r, d, d)
        cr.stroke()

        dr = (1 - math.sin(math.pi / 4)) * r
        cr.move_to(dr, dr)
        cr.line_to(d - dr, d - dr)
        cr.move_to(dr, d - dr)
        cr.line_to(d - dr, dr)
        cr.stroke()
        
        super(FlowFinalNodeItem, self).draw(context)
        


class DecisionNodeItem(ActivityNodeItem):
    """
    Representation of decision or merge node.
    """
    __uml__   = UML.DecisionNode
    __style__   = {
        'min-size':   (20, 30),
        'name-align': (ALIGN_LEFT, ALIGN_TOP),
    }

    RADIUS = 15

    def __init__(self, id=None):
        ActivityNodeItem.__init__(self, id)
        self._combined = None
        #self.set_prop_persistent('combined')

    def save(self, save_func):
        if self._combined:
            save_func('combined', self._combined, reference=True)
        super(DecisionNodeItem, self).save(save_func)

    def load(self, name, value):
        if name == 'combined':
            self._combined = value
        else:
            super(DecisionNodeItem, self).load(name, value)

    @observed
    def _set_combined(self, value):
        #self.preserve_property('combined')
        self._combined = value

    combined = reversible_property(lambda s: s._combined, _set_combined)
        
    def draw(self, context):
        """
        Draw diamond shape, which represents decision and merge nodes.
        """
        cr = context.cairo
        r = self.RADIUS
        r2 = r * 2/3

        cr.move_to(r2, 0)
        cr.line_to(r2 * 2, r)
        cr.line_to(r2, r * 2)
        cr.line_to(0, r)
        cr.close_path()
        cr.stroke()

        super(DecisionNodeItem, self).draw(context)



class ForkNodeItem(Item, DiagramItem):
    """
    Representation of fork and join node.
    """
    __namedelement__ = True

    element_factory = inject('element_factory')

    __uml__   = UML.ForkNode

    __style__ = {
        'min-size':   (6, 45),
        'name-align': (ALIGN_CENTER, ALIGN_BOTTOM),
        'name-padding': (2, 2, 2, 2),
        'name-outside': True,
    }

    STYLE_TOP = {
        'text-align': (ALIGN_CENTER, ALIGN_TOP),
        'text-outside': True,
    }

    def __init__(self, id=None):
        Item.__init__(self)
        DiagramItem.__init__(self, id)
        
        self._handles.extend((Handle(), Handle()))

        self._combined = None
        self._constraints = []

        self._join_spec = self.add_text('joinSpec.value',
            pattern='{ joinSpec = %s }',
            style=self.STYLE_TOP,
            visible=self.is_join_spec_visible)


    def save(self, save_func):
        save_func('matrix', tuple(self.matrix))
        save_func('height', float(self._handles[1].y))
        if self._combined:
            save_func('combined', self._combined, reference=True)
        DiagramItem.save(self, save_func)

    def load(self, name, value):
        if name == 'matrix':
            self.matrix = eval(value)
        elif name == 'height':
            self._handles[1].y = eval(value)
        elif name == 'combined':
            self._combined = value
        else:
            #DiagramItem.load(self, name, value)
            super(ForkNodeItem, self).load(name, value)

    def postload(self):
        subject = self.subject
        if subject and isinstance(subject, UML.JoinNode) and subject.joinSpec:
            self._join_spec.text = self.subject.joinSpec.value
        super(ForkNodeItem, self).postload()

    @observed
    def _set_combined(self, value):
        #self.preserve_property('combined')
        self._combined = value

    combined = reversible_property(lambda s: s._combined, _set_combined)

    def setup_canvas(self):
        Item.setup_canvas(self)

        h1, h2 = self._handles
        cadd = self.canvas.solver.add_constraint
        c1 = EqualsConstraint(a=h1.x, b=h2.x)
        c2 = LessThanConstraint(smaller=h1.y, bigger=h2.y, delta=30)
        self._constraints.extend((cadd(c1), cadd(c2)))


    def teardown_canvas(self):
        super(ForkNodeItem, self).teardown_canvas()
        for c in self._constraints:
            self.canvas.solver.remove_constraint(c)


    def is_join_spec_visible(self):
        """
        Check if join specification should be displayed.
        """
        return isinstance(self.subject, UML.JoinNode) \
            and self.subject.joinSpec is not None \
            and self.subject.joinSpec.value != DEFAULT_JOIN_SPEC


    def text_align(self, extents, align, padding, outside):
        h1, h2 = self._handles
        w, _ = self.style.min_size
        h = h2.y - h1.y
        x, y = get_text_point(extents, w, h, align, padding, outside)

        return x, y


    def pre_update(self, context):
        self.update_stereotype()
        Item.pre_update(self, context)
        DiagramItem.pre_update(self, context)


    def post_update(self, context):
        Item.post_update(self, context)
        DiagramItem.post_update(self, context)


    def draw(self, context):
        """
        Draw vertical line - symbol of fork and join nodes. Join
        specification is also drawn above the item.
        """
        Item.draw(self, context)
        DiagramItem.draw(self, context)

        cr = context.cairo

        cr.set_line_width(6)
        h1, h2 = self._handles
        cr.move_to(h1.x, h1.y)
        cr.line_to(h2.x, h2.y)

        cr.stroke()


    def point(self, x, y):
        h1, h2 = self._handles
        d, p = distance_line_point(h1.pos, h2.pos, (x, y))
        # Substract line_width / 2
        return d - 3


    def on_subject_notify(self, pspec, notifiers = ()):
        """
        Detect changes of subject.

        If subject is join node, then set subject of join specification
        text element.
        """
        subject = self.subject
        if is_join_node(subject):
            join_spec_notifiers = ('joinSpec', 'joinSpec.value')
        else:
            join_spec_notifiers = ()

        DiagramItem.on_subject_notify(self, pspec, join_spec_notifiers + notifiers)

        if is_join_node(subject) and not (subject.joinSpec and subject.joinSpec.value):
            self.set_join_spec(DEFAULT_JOIN_SPEC)
        self.request_update()


    def set_join_spec(self, value):
        """
        Set join specification.
        """
        subject = self.subject
        if not is_join_node(subject):
            return

        if not subject.joinSpec:
            subject.joinSpec = self.element_factory.create(UML.LiteralSpecification)

        if not value:
            value = DEFAULT_JOIN_SPEC

        subject.joinSpec.value = value
        self._join_spec.text = value


    def on_subject_notify__joinSpec(self, subject, pspec=None):
        self.request_update()


    def on_subject_notify__joinSpec_value(self, subject, pspec=None):
        self.request_update()


def is_join_node(subject):
    """
    Check if ``subject`` is join node. 
    """
    return subject and isinstance(subject, UML.JoinNode)

# vim:sw=4:et



"""
Setup script for Gaphor.

Run 'python setup.py develop' to set up a development environment, including
dependencies.

Run 'python setup.py run' to start Gaphor directly (without install).
"""

VERSION = '0.11.2'

import os
import sys
sys.path.insert(0, '.')

from ez_setup import use_setuptools

use_setuptools()

from setuptools import setup, find_packages
from distutils.cmd import Command

from utils.command.build_mo import build_mo
from utils.command.build_pot import build_pot
from utils.command.build_uml import build_uml
from utils.command.install_lib import install_lib
from utils.command.run import run

LINGUAS = [ 'ca', 'es', 'nl', 'sv' ]

class build_doc(Command):
    description = 'Builds the documentation'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
#        from docutils.core import publish_cmdline
#        docutils_conf = os.path.join('doc', 'docutils.conf')
        epydoc_conf = os.path.join('doc', 'epydoc.conf')

#        for source in glob('doc/*.txt'):
#            dest = os.path.splitext(source)[0] + '.html'
#            if not os.path.exists(dest) or \
#                   os.path.getmtime(dest) < os.path.getmtime(source):
#                print 'building documentation file %s' % dest
#                publish_cmdline(writer_name='html',
#                                argv=['--config=%s' % docutils_conf, source,
#                                      dest])

        try:
            from epydoc import cli
            old_argv = sys.argv[1:]
            sys.argv[1:] = [
                '--config=%s' % epydoc_conf,
                '--no-private', # epydoc bug, not read from config
                '--simple-term',
                '--verbose'
            ]
            cli.cli()
            sys.argv[1:] = old_argv

        except ImportError:
            print 'epydoc not installed, skipping API documentation.'


#if sys.platform == 'darwin':
#    # Mac OS X
#    import pkg_resources
#    pkg_resources.require('zope.component')
#    platform_setup_requires=['py2app']
#    platform_setup = dict(
#        app=['gaphor-osx.py'],
#        )
#else:
platform_setup_requires = []
platform_setup = dict()


setup(
    name='gaphor',
    version=VERSION,
    url='http://gaphor.devjavu.com',
    author='Arjan J. Molenaar',
    author_email='arjanmol@users.sourceforge.net',
    license='GNU General Public License',
    description='Gaphor is a UML modeling tool',
    long_description="""
Gaphor is a UML modeling tool written in Python.

It uses the GTK+ environment for user interaction.
""",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: X11 Applications :: GTK',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Topic :: Multimedia :: Graphics :: Editors :: Vector-Based',
        'Topic :: Software Development :: Documentation',
    ],

    keywords = 'model modeling modelling uml diagram python tool',

    packages = find_packages(exclude=['ez_setup', 'utils*']),

    include_package_data = True,

    install_requires = [
        # 'PyGTK >= 2.8.0', - Exclude, since it will not build anyway
        'gaphas >= 0.3.0',
        'zope.component >= 3.3.0', # - won't compile on windows.
        # Add dependency on zope.testing to work around bug in zope.component
        'zope.testing >= 3.3.0',
    ],

    zip_safe = False,

    #test_suite = 'nose.collector',

    entry_points = {
        'console_scripts': [
            'gaphor = gaphor:main',
        ],
        'gaphor.services': [
            'adapter_loader = gaphor.services.adapterloader:AdapterLoader',
            'properties = gaphor.services.properties:Properties',
            'undo_manager = gaphor.services.undomanager:UndoManager',
            'element_factory = gaphor.UML.elementfactory:ElementFactory',
            'file_manager = gaphor.services.filemanager:FileManager',
            'diagram_export_manager = gaphor.services.diagramexportmanager:DiagramExportManager',
            'action_manager = gaphor.services.actionmanager:ActionManager',
            'gui_manager = gaphor.services.guimanager:GUIManager',
            'copy = gaphor.services.copyservice:CopyService',
            'xmi_export = gaphor.plugins.xmiexport:XMIExport',
            'diagram_layout = gaphor.plugins.diagramlayout:DiagramLayout',
            'pynsource = gaphor.plugins.pynsource:PyNSource',
            'check_metamodel = gaphor.plugins.checkmetamodel:CheckModelWindow',
            'live_object_browser = gaphor.plugins.liveobjectbrowser:LiveObjectBrowser',
            'alignment = gaphor.plugins.alignment:Alignment',
            'help = gaphor.services.helpservice:HelpService',
        ],
        'gaphor.uicomponents': [
            'mainwindow = gaphor.ui.mainwindow:MainWindow',
            'consolewindow = gaphor.ui.consolewindow:ConsoleWindow',
        ],
        'distutils.commands': [
            'nosetests = nose.commands:nosetests',
        ],
    },

    cmdclass = {
              'build_uml': build_uml,
              'build_mo': build_mo,
              'build_pot': build_pot,
              'build_doc': build_doc,
              'install_lib': install_lib,
              'run': run,
    },

    setup_requires = ['nose >= 0.9.2'] + platform_setup_requires,

    test_suite = 'nose.collector',

    options = dict(
        py2app = dict(
            argv_emulation=True,
            semi_standalone=True, # Depend on installed Python 2.4 Framework
            includes=['atk', 'pango', 'cairo', 'pangocairo'], #'zope.defferedimport', 'zope.component', 'zope.deprecation', 'zope.interface', 'zope.event', 'zope.testing', 'zope.proxy'],
            packages=['gaphor', 'zope'],
            plist=dict(
                CFBundleGetInfoString='Gaphor',
                CFBundleIdentifier='com.devjavu.gaphor'
                )
        ),
        build_pot = dict(
            all_linguas = ','.join(LINGUAS),
        ),
        build_mo = dict(
            all_linguas = ','.join(LINGUAS),
        ),

    ),

    **platform_setup

)
 
# vim:sw=4:et:ai



"""
Plugin based on the Live Object browser
(http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/300304).
It shows the state of the data model at the time the browser is activated.
"""

__version__ = '0.1'
__author__ = 'Arjan Molenaar'

from zope import interface
from gaphor.core import _, inject, action, build_action_group
from gaphor.interfaces import IService, IActionProvider
from browser import Browser


class LiveObjectBrowser(object):

    interface.implements(IService, IActionProvider)

    element_factory = inject('element_factory')

    menu_xml = """
      <ui>
        <menubar action="mainwindow">
          <menu action="tools">
            <menuitem action="tools-life-object-browser" />
          </menu>
        </menubar>
      </ui>"""

    def __init__(self):
        self.action_group = build_action_group(self)

    def init(self, app):
        pass

    def shutdown(self):
        pass

    @action(name='tools-life-object-browser', label='Life object browser')
    def execute(self):
        browser = Browser("resource", self.element_factory.lselect())


# vim:sw=4:et



# vim:sw=4:et

import sys
from xml.sax.saxutils import XMLGenerator

class XMLWriter(XMLGenerator):
    """An XMLGenerator with CDATA handling extensions.
    """

    def __init__(self, out=None):
        XMLGenerator.__init__(self, out, sys.getdefaultencoding())
        self._in_cdata = False

    def characters(self, content):
        if self._in_cdata:
            self._out.write(content.replace(']]>', '] ]>'))
        else:
            XMLGenerator.characters(self, content)

    # Lexical handler methods

    def comment(self, comment):
        self._out.write('<!-- ')
        self._out.write(comment.replace('-->', '- ->'))
        self._out.write(' -->')

    def startCDATA(self):
        self._out.write('<![CDATA[')
        self._in_cdata = True

    def endCDATA(self):
        self._out.write(']]>')
        self._in_cdata = False

